# This file was autogenerated! Do not modify!
# set interfacing Set.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class SetInt(PmObject):
    cdef pm_Set[int] pm_obj

cdef class SetMatrixRational(PmObject):
    cdef pm_Set[pm_Matrix[pm_Rational]] pm_obj

cdef class SetArrayInt(PmObject):
    cdef pm_Set[pm_Array[int]] pm_obj

cdef class SetSetInt(PmObject):
    cdef pm_Set[pm_Set[int]] pm_obj



###############################################################################
#       Copyright (C) 2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr> 
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from libcpp cimport bool
from libcpp.string cimport string

cdef class RationalFunctionRationalInt(PmObject):
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')



#!/usr/bin/env python

import unittest

# NOTE: apparently this has to be done *before* importing polymake.
# Otherwise it produces segfault...
try:
    import sage.all
    import sage
except:
    sage = None

import polymake

class Examples:
    @staticmethod
    def p1():
        m1 = [[1,0,0,0], [1,0,0,1], [1,0,1,0], [1,0,1,1],  [1,1,0,0], [1,1,0,1], [1,1,1,0], [1,1,1,1]]
        return polymake.Polytope(POINTS=m1)

    @staticmethod
    def p2():
        m2 = [[1,0,0,0], [1,(1,16),(1,4),(1,16)], [1,(3,8),(1,4),(1,32)],
              [1,(1,4),(3,8),(1,32)], [1,(1,16),(1,16),(1,4)], [1,(1,32),(3,8),(1,4)],
              [1,(1,4),(1,16),(1,16)], [1,(1,32),(1,4),(3,8)], [1,(3,8),(1,32),(1,4)],
              [1,(1,4),(1,32),(3,8)]]
        return polymake.Polytope(POINTS=m2)

    @staticmethod
    def p3():
        m3 = [[1, 3, 0, 0], [1, 0, 3, 0], [1, 1, 1, 1], [1, 0, 0, 3]]
        return polymake.Polytope(POINTS=m3)

    @staticmethod
    def p4():
        m4 = [[(-20, 9), (-19, 1), (-8, 7), (-11, 13), (-9, 20)],
              [(-13, 1), (1, 2), (20, 13), (5, 2), (-19, 12)],
              [(-2, 3), (-9, 8), (-11, 13), (2, 1), (-4, 9)],
              [(-5, 4), (-18, 7), (5, 2), (-3, 2), (-3, 5)],
              [(-4, 3), (-7, 11), (-17, 3), (-5, 11), (-1, 9)],
              [(6, 1), (14, 19), (2, 11), (-5, 4), (-13, 2)],
              [(-3, 2), (-1, 4), (-15, 19), (-5, 1), (19, 2)],
              [(-4, 5), (1, 3), (4, 5), (3, 8), (9, 1)],
              [(-8, 9), (-4, 5), (1, 5), (17, 6), (-3, 1)],
              [(-3, 19), (-15, 1), (11, 2), (5, 19), (-2, 1)],
              [(4, 1), (-5, 7), (-6, 1), (-5, 4), (3, 5)]]
        return polymake.Polytope(POINTS=m4)

    @staticmethod
    def p5():
        # one vertex and two rays
        return polymake.Polytope(POINTS=[[1,0,0,0], [0,0,1,0], [0,1,0,0]])

examples = Examples()

class TestPolymakePolytope(unittest.TestCase):
    def test_to_sage(self):
        if sage is None:
            return
        examples.p1().sage()
        examples.p2().sage()
        examples.p3().sage()
        examples.p4().sage()

        q5 = examples.p5().sage()
        self.assertEqual(q5.n_vertices(), 1)
        self.assertEqual(q5.n_rays(), 2)
        self.assertEqual(q5.n_lines(), 0)

    def test_from_sage(self):
        if sage is None:
            return

        C = sage.all.polytopes.hypercube(4)
        P = polymake.polymake(C)
        self.assertEqual(P.F_VECTOR.python(), list(C.f_vector())[1:-1])

    def test_AFFINE_HULL(self):
        a3 = examples.p3().AFFINE_HULL
        self.assertEqual(a3[0,0], -3)
        self.assertEqual(a3[0,1], 1)
        self.assertEqual(a3[0,2], 1)
        self.assertEqual(a3[0,3], 1)

    def test_ALTSHULER_DET(self):
        self.assertEqual(examples.p4().ALTSHULER_DET, 0)
        self.assertEqual(examples.p2().ALTSHULER_DET, 1872)
        self.assertEqual(examples.p3().ALTSHULER_DET, 4)

    def test_BALANCE(self):
        self.assertEqual(examples.p1().BALANCE, 1)
        self.assertEqual(examples.p2().BALANCE, 1)
        self.assertEqual(examples.p3().BALANCE, 2)

    def test_BALANCED(self):
        self.assertTrue(examples.p1().BALANCED)
        self.assertTrue(examples.p2().BALANCED)
        self.assertTrue(examples.p3().BALANCED)

    def test_BOUNDED(self):
        self.assertTrue(examples.p1().BOUNDED)
        self.assertTrue(examples.p2().BOUNDED)
        self.assertTrue(examples.p3().BOUNDED)

    def test_CD_INDEX_COEFFICIENTS(self):
        # for now, jut checking that we do not get errors
        examples.p1().CD_INDEX_COEFFICIENTS
        examples.p2().CD_INDEX_COEFFICIENTS
        examples.p3().CD_INDEX_COEFFICIENTS
        polymake.cube(3).CD_INDEX_COEFFICIENTS

    def test_CENTERED(self):
        self.assertFalse(examples.p1().CENTERED)
        self.assertFalse(examples.p2().CENTERED)
        self.assertFalse(examples.p3().CENTERED)

    def test_CENTRALLY_SYMMETRIC(self):
        self.assertFalse(examples.p1().CENTRALLY_SYMMETRIC)
        self.assertFalse(examples.p2().CENTRALLY_SYMMETRIC)
        self.assertFalse(examples.p3().CENTRALLY_SYMMETRIC)

# broken in last version!!
#    def test_CENTROID(self):
#        # for now, just checking that we do not get errors
#        examples.p1().CENTROID
#        examples.p2().CENTROID
#        # got an error with p3
#        polymake.cube(3).CENTROID
#        polymake.cube(4).CENTROID

    def test_COCUBICAL(self):
        self.assertFalse(examples.p1().COCUBICAL)
        self.assertFalse(examples.p2().COCUBICAL)
        self.assertTrue(examples.p3().COCUBICAL)

    def test_COCIRCUIT_EQUATIONS(self):
        m = examples.p1().COCIRCUIT_EQUATIONS
        self.assertEqual(m[0,0], 0)
        self.assertEqual(m[0,2], 1)
        self.assertEqual(m[0,8], -1)
        examples.p2().COCIRCUIT_EQUATIONS
        examples.p3().COCIRCUIT_EQUATIONS

    def test_COMBINATORIAL_DIM(self):
        self.assertEqual(examples.p1().COMBINATORIAL_DIM, 3)
        self.assertEqual(examples.p2().COMBINATORIAL_DIM, 3)
        self.assertEqual(examples.p3().COMBINATORIAL_DIM, 2)

    def test_COMPLEXITY(self):
        c3 = polymake.cube(3)
        c4 = polymake.cube(4)
        with self.assertRaises(RuntimeError):
            c3.COMPLEXITY
        comp = c4.COMPLEXITY
        self.assertLess(comp, 3.14286)
        self.assertGreater(comp, 3.14285)

    def test_CS_PERMUTATION(self):
        with self.assertRaises(ValueError):
            examples.p1().CS_PERMUTATION
        polymake.cube(3).CS_PERMUTATION

    def test_CUBICAL(self):
        self.assertTrue(examples.p1().CUBICAL)
        self.assertFalse(examples.p2().CUBICAL)
        self.assertTrue(examples.p3().CUBICAL)
        self.assertTrue(polymake.cube(3).CUBICAL)

    def test_CUBICAL_H_VECTOR(self):
        v = examples.p1().CUBICAL_H_VECTOR
        self.assertEqual(v[0], 4)
        self.assertEqual(v[1], 4)
        self.assertEqual(v[2], 4)
        self.assertEqual(v[3], 4)

        v = examples.p3().CUBICAL_H_VECTOR
        self.assertEqual(v[0], 2)
        self.assertEqual(v[1], 1)
        self.assertEqual(v[2], 2)

    def test_CUBICALITY(self):
        self.assertEqual(examples.p1().CUBICALITY, 3)
        self.assertEqual(examples.p2().CUBICALITY, 1)
        self.assertEqual(examples.p3().CUBICALITY, 1)

# BUG: Segfaults
#    def test_DEGREE_ONE_GENERATORS(self):
#        # for now, just checking that we do not get errors
#        examples.p1().DEGREE_ONE_GENERATORS
#        examples.p2().DEGREE_ONE_GENERATORS
#        examples.p3().DEGREE_ONE_GENERATORS
#        polymake.cube(3).DEGREE_ONE_GENERATORS
#        polymake.cube(4).DEGREE_ONE_GENERATORS

    def test_DUAL_BOUNDED_H_VECTOR(self):
        v = examples.p1().DUAL_BOUNDED_H_VECTOR
        self.assertEqual(v[0], 1)
        self.assertEqual(v[1], 3)
        self.assertEqual(v[2], 3)
        self.assertEqual(v[3], 1)

        v = examples.p3().DUAL_BOUNDED_H_VECTOR
        self.assertEqual(v[0], 1)
        self.assertEqual(v[1], 1)
        self.assertEqual(v[2], 1)

    def test_DUAL_GRAPH(self):
        for pol in [examples.p1(), examples.p2(), examples.p3(),
                polymake.cube(3), polymake.cube(4)]:
            g = pol.DUAL_GRAPH
            self.assertTrue(g.type_name().startswith('Graph<Undirected>'))

    def test_EDGE_ORIENTABLE(self):
        self.assertEqual(examples.p1().EDGE_ORIENTABLE, True)
# BUG: now a RuntimeError
#        with self.assertRaises(ValueError):
#            examples.p2().EDGE_ORIENTABLE
#        with self.assertRaises(ValueError):
#            examples.p3().EDGE_ORIENTABLE
        self.assertTrue(polymake.cube(3).EDGE_ORIENTABLE, True)
        self.assertTrue(polymake.cube(4).EDGE_ORIENTABLE, True)

    def test_EDGE_ORIENTATION(self):
        m = examples.p1().EDGE_ORIENTATION
        with self.assertRaises(ValueError):
            examples.p2().EDGE_ORIENTATION
        with self.assertRaises(ValueError):
            examples.p3().EDGE_ORIENTATION
        m = polymake.cube(3).EDGE_ORIENTATION
        m = polymake.cube(4).EDGE_ORIENTATION

# BUG: segfault...
#    def test_EHRHART_POLYNOMIAL_COEFF(self):
#        for pol in [examples.p1(), examples.p3(),
#                polymake.cube(3), polymake.cube(4)]:
#            v = pol.EHRHART_POLYNOMIAL_COEFF

    def test_ESSENTIALLY_GENERIC(self):
        self.assertFalse(examples.p1().ESSENTIALLY_GENERIC)
        self.assertFalse(examples.p2().ESSENTIALLY_GENERIC)
        self.assertTrue(examples.p3().ESSENTIALLY_GENERIC)
        self.assertFalse(polymake.cube(3).ESSENTIALLY_GENERIC)
        self.assertFalse(polymake.cube(4).ESSENTIALLY_GENERIC)

    def test_N_FACETS(self):
        self.assertEqual(examples.p1().N_FACETS, 6)
        self.assertEqual(examples.p2().N_FACETS, 13)
        self.assertEqual(examples.p3().N_FACETS, 3)

    def test_N_VERTICES(self):
        self.assertEqual(examples.p1().N_VERTICES, 8)
        self.assertEqual(examples.p2().N_VERTICES, 10)
        self.assertEqual(examples.p3().N_VERTICES, 3)

    def test_SIMPLE(self):
        self.assertTrue(examples.p1().SIMPLE)
        self.assertFalse(examples.p2().SIMPLE)
        self.assertTrue(examples.p3().SIMPLE)

    def test_SIMPLICIAL(self):
        self.assertFalse(examples.p1().SIMPLICIAL)
        self.assertFalse(examples.p2().SIMPLICIAL)
        self.assertTrue(examples.p3().SIMPLICIAL)

    def test_F_VECTOR(self):
        self.assertEqual(examples.p1().F_VECTOR.python(), [8, 12, 6])
        self.assertEqual(examples.p2().F_VECTOR.python(), [10, 21, 13])
        self.assertEqual(examples.p3().F_VECTOR.python(), [3, 3])

if __name__ == '__main__':
    unittest.main()



# This file was autogenerated! Do not modify!
# quadratic_extension interfacing QuadraticExtension.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class QuadraticExtension(PmObject):
    cdef pm_QuadraticExtension pm_obj



from __future__ import absolute_import

import glob
from os.path import join, exists, getmtime

from .pm_types import pm_modules

from .handlers import (write_handlers,
        write_mappings,
        write_declarations,
        write_definitions,
#        write_undefined_classes
        )

def rebuild(force=False):
    module_path = 'polymake'
    print("*** Rebuilding handlers and mappings ***")

    src_files = glob.glob(join('autogen', '*.py'))
    gen_files = [join(module_path, "auto_handlers.pxi"),
                 join(module_path, "auto_mappings.pxi")] + \
                [join(module_path, '{}.pxd'.format(mod)) for mod in pm_modules()]

    if not force and all(exists(f) for f in gen_files):
        src_mtime = max(getmtime(f) for f in src_files)
        gen_mtime = max(getmtime(f) for f in gen_files)

        if gen_mtime > src_mtime:
            return

    write_handlers(join(module_path, "auto_handlers.pxi"))
    write_mappings(join(module_path, "auto_mappings.pxi"))
    write_declarations()
#    write_undefined_classes()



#!/usr/bin/env python

import unittest
import random

# NOTE: apparently this has to be done *before* importing polymake.
# Otherwise it produces segfault...
try:
    import sage.all
    import sage
except ImportError:
    sage = None

import polymake

try:
    import gmpy2
except ImportError:
    gmpy2 = None

class TestPolymakeInteger(unittest.TestCase):
    def test_init(self):
        for a in [-1,0,1, 2**32, 2**32-1, 2**32+1,
                2**64, 2**64-1, 2**64+1, -2**32, -2**32-1, -2**32+1,
                -2**64, -2**64-1, -2**64+1]:
            polymake.Integer(a)

    def test_bool(self):
        self.assertEqual(bool(polymake.Integer(0)), False)
        self.assertEqual(bool(polymake.Integer(1)), True)
        self.assertEqual(bool(polymake.Integer(-1)), True)
        self.assertEqual(bool(polymake.Integer(2)), True)
        self.assertEqual(bool(polymake.Integer(-2)), True)

    def test_idempotent(self):
        for a in [0, -2, 2**100+3]:
            b = polymake.Integer(a)
            self.assertEqual(polymake.Integer(b), b)

    def test_sage(self):
        if sage is None:
            return

        for a in [0, -2, 2**100+3]:
            b1 = polymake.Integer(a).sage()
            b2 = sage.all.Integer(polymake.Integer(a))
            c = sage.all.Integer(a)
            self.assertEqual(type(b1), type(c))
            self.assertEqual(type(b2), type(c))
            self.assertEqual(b1, c)
            self.assertEqual(b2, c)

    def test_gmpy2(self):
        if gmpy2 is None:
            return

        for a in [-2, 2**100+3]:
            b = gmpy2.mpz(polymake.Integer(a))
            c = gmpy2.mpz(a)
            self.assertEqual(type(b), type(c))
            self.assertEqual(b, c)

    def test_index(self):
        self.assertTrue([0,1,5,3][polymake.Integer(2)], 5)

    def test_num_den(self):
        one = polymake.Integer(1)
        for _ in range(10):
            a = polymake.Integer(random.randint(-2**100, 2**100))
            self.assertEqual(a.numerator(), a)
            self.assertEqual(a.denominator(), one)

    def test_int(self):
        for a in [1, -2, 2**100+3]:
            self.assertEqual(int(polymake.Integer(a)), a)

    def test_float(self):
        for a in [0, 1, -4]:
            self.assertEqual(float(polymake.Integer(a)), float(a))

    def test_big(self):
        a = 3**100
        b = polymake.Integer(a)
        self.assertEqual(a, b.__index__())

    def test_pow(self):
        for _ in range(20):
            a = random.randint(-100, 100)
            b = random.randint(0, 10)

            ans = polymake.Integer(a**b)
            self.assertEqual(polymake.Integer(a)**b, ans)
            self.assertEqual(polymake.Integer(a)**polymake.Integer(b), ans)

        with self.assertRaises(ValueError):
            polymake.Integer(2) ** (-3)
        with self.assertRaises(OverflowError):
            polymake.Integer(2) ** (2**100)

    def test_pow_other(self):
        for _ in range(20):
            a = random.randint(-100, 100)
            b = random.randint(0, 10)

            B = [b]
            if sage is not None:
                B.append(sage.all.Integer(b))
            if gmpy2 is not None:
                B.append(gmpy2.mpz(b))

            ans = polymake.Integer(a**b)
            for bb in B:
                self.assertTrue(isinstance(polymake.Integer(a)**bb, polymake.Integer))
                self.assertEqual(polymake.Integer(a)**bb, ans)

        with self.assertRaises(ValueError):
            polymake.Integer(2) ** (-3)
        with self.assertRaises(OverflowError):
            polymake.Integer(2) ** (2**100)


    def test_cmp(self):
        for _ in range(10):
            a = random.randint(-10, 10)
            b = a + random.randint(1, 10)
            pa = polymake.Integer(a)
            pb = polymake.Integer(b)
            self.assertTrue(pa == pa and a == pa and pa == a)
            self.assertTrue(pa != pb and a != pb and pa != b)
            self.assertTrue(pa <  pb and a <  pb and pa <  b)
            self.assertTrue(pa <= pb and a <= pb and pa <= b)

            self.assertTrue(pb >= pa and b >= pa and pb >= a)
            self.assertTrue(pb >  pa and b >  pa and pb >  a)

            self.assertFalse(pa != pa or a != pa or pa != a)
            self.assertFalse(pa == pb or a == pb or pa == b)
            self.assertFalse(pa >= pb or a >= pb or pa >= b)
            self.assertFalse(pa >  pb or a >  pb or pa >  b)

            self.assertFalse(pb <= pa or b <= pa or pb <= a)
            self.assertFalse(pb <  pa or b <  pa or pb <  a)

    def test_cmp_other(self):
        for _ in range(20):
            a = random.randint(-2**100, 2**100)
            b = a + random.randint(1, 2**100)
            pa = polymake.Integer(a)
            pb = polymake.Integer(b)
            A = [a]
            B = [b]
            if sage is not None:
                A.append(sage.all.Integer(a))
                B.append(sage.all.Integer(b))
            if gmpy2 is not None:
                A.append(gmpy2.mpz(a))
                B.append(gmpy2.mpz(b))
            for aa in A:
                self.assertTrue(aa == pa and aa <= pa and pa <= aa and aa >= pa and pa >= aa)
                self.assertFalse(aa != pa or aa < pa or aa > pa)
            for bb in B:
                self.assertTrue(pa < bb and bb > pa and pa != bb and bb != pa)
                self.assertFalse(pa >= bb or bb <= pa or pa == bb or bb == pa)

    def test_binop(self):
        for _ in range(30):
            a = random.randint(-100, 100)
            b = random.randint(-100, 100)
            pa = polymake.Integer(a)
            pb = polymake.Integer(b)

            s = polymake.Integer(a+b)
            p = polymake.Integer(a*b)
            d = polymake.Integer(a-b)
            if b:
                q = polymake.Integer(a // b)
            else:
                q = None

            A = [a]
            B = [b]
            if sage is not None:
                A.append(sage.all.Integer(a))
                B.append(sage.all.Integer(b))
            if gmpy2 is not None:
                A.append(gmpy2.mpz(a))
                B.append(gmpy2.mpz(b))

            # Integer op Integer
            self.assertEqual(pa+pb, s)
            self.assertEqual(pa*pb, p)
            self.assertEqual(pa-pb, d)
            # TODO: don't agree for negative values
            if a >= 0 and b > 0:
                self.assertEqual(pa//pb, q)

            # Integer op int
            for bb in B:
                self.assertEqual(pa+bb, s)
                self.assertEqual(pa*bb, p)
                self.assertEqual(pa-bb, d)
                # TODO: don't agree for negative values
                if a >= 0 and b > 0:
                    self.assertEqual(pa//bb, q)

            # int op Integer
            for aa in A:
                self.assertEqual(aa+pb, s)
                self.assertEqual(aa*pb, p)
                self.assertEqual(aa-pb, d)
                # TODO: don't agree for negative values
                if a >= 0 and b > 0:
                    self.assertEqual(aa//pb, q)

    def test_neg(self):
        for _ in range(10):
            a = random.randint(-10, 10)
            pa = polymake.Integer(a)
            self.assertEqual(-pa, polymake.Integer(-a))

    def test_zero_division(self):
        zeros = [0, polymake.Integer(0), polymake.rational.Rational(0)]
        if sage is not None:
            zeros.append(sage.all.Integer(0))
            zeros.append(sage.all.Rational(0))
        if gmpy2 is not None:
            zeros.append(gmpy2.mpz(0))
            zeros.append(gmpy2.mpq(0))
        rats = [polymake.Integer(0), polymake.Integer(1)]
        if sage is not None:
            rats.append(sage.all.Integer(1))
        for z in zeros:
            for r in rats:
                with self.assertRaises(ZeroDivisionError):
                    r / z
                with self.assertRaises(ZeroDivisionError):
                    r // z

if __name__ == '__main__':
    unittest.main()



"""
Various functions to deal with conversion mpz <-> Python int/long

AUTHORS:

- Gonzalo Tornaria (2006): initial version

- David Harvey (2007-08-18): added ``mpz_get_pyintlong`` function
  (:trac:`440`)

- Jeroen Demeyer (2015-02-24): moved from c_lib, rewritten using
  ``mpz_export`` and ``mpz_import`` (:trac:`17853`)

- Vincent Delecroix (2016)
"""

#*****************************************************************************
#       Copyright (C) 2015 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#                     2016 Vincent Delecroix <vincent.delecroix@labri.fr>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from __future__ import absolute_import, print_function

from cysignals.signals cimport sig_on, sig_off

from libc.stdlib cimport malloc, free
from libc.math cimport ldexp
from libc.stdint cimport uint64_t

from cpython.object cimport Py_SIZE
from cpython.int cimport PyInt_FromLong
from cpython.long cimport PyLong_FromLong

# NOTE: needs recent enough Cython
from cpython.longintrepr cimport _PyLong_New, py_long, digit, PyLong_SHIFT

from .mpz cimport *
from .mpq cimport *

cdef extern from *:
    Py_ssize_t* Py_SIZE_PTR "&Py_SIZE"(object)
    int hash_bits """
        #ifdef _PyHASH_BITS
        _PyHASH_BITS         /* Python 3 */
        #else
        (8 * sizeof(void*))  /* Python 2 */
        #endif
        """
    int limb_bits "(8 * sizeof(mp_limb_t))"

# Unused bits in every PyLong digit
cdef size_t PyLong_nails = 8*sizeof(digit) - PyLong_SHIFT

cdef mpz_get_pylong_large(mpz_srcptr z):
    """
    Convert a non-zero ``mpz`` to a Python ``long``.
    """
    cdef size_t nbits = mpz_sizeinbase(z, 2)
    cdef size_t pylong_size = (nbits + PyLong_SHIFT - 1) // PyLong_SHIFT
    L = _PyLong_New(pylong_size)
    mpz_export(L.ob_digit, NULL,
            -1, sizeof(digit), 0, PyLong_nails, z)
    if mpz_sgn(z) < 0:
        # Set correct size (use a pointer to hack around Cython's
        # non-support for lvalues).
        sizeptr = Py_SIZE_PTR(L)
        sizeptr[0] = -pylong_size
    return L


cdef mpz_get_pylong(mpz_srcptr z):
    """
    Convert an ``mpz`` to a Python ``long``.
    """
    if mpz_fits_slong_p(z):
        return PyLong_FromLong(mpz_get_si(z))
    return mpz_get_pylong_large(z)


cdef mpz_get_pyintlong(mpz_srcptr z):
    """
    Convert an ``mpz`` to a Python ``int`` if possible, or a ``long``
    if the value is too large.
    """
    if mpz_fits_slong_p(z):
        return PyInt_FromLong(mpz_get_si(z))
    return mpz_get_pylong_large(z)


cdef int mpz_set_pylong(mpz_ptr z, L) except -1:
    """
    Convert a Python ``long`` `L` to an ``mpz``.
    """
    cdef Py_ssize_t pylong_size = Py_SIZE(L)
    if pylong_size < 0:
        pylong_size = -pylong_size
    mpz_import(z, pylong_size, -1, sizeof(digit), 0, PyLong_nails,
            (<py_long>L).ob_digit)
    if Py_SIZE(L) < 0:
        mpz_neg(z, z)


cdef Py_hash_t mpz_pythonhash(mpz_srcptr z):
    """
    Hash an ``mpz``, where the hash value is the same as the hash value
    of the corresponding Python ``int`` or ``long``, except that we do
    not replace -1 by -2 (the Cython wrapper for ``__hash__`` does that).
    """
    if mpz_sgn(z) == 0:
        return 0

    # The hash value equals z % m where m = 2 ^ hash_bits - 1.
    #
    # Safely compute 2 ^ hash_bits - 1 without overflow
    cdef mp_limb_t modulus = (((<mp_limb_t>(1) << (hash_bits - 1)) - 1) * 2) + 1

    cdef mp_limb_t h = 0
    cdef mp_limb_t x, y
    cdef size_t i, n
    cdef unsigned int r
    n = mpz_size(z)
    for i in range(n):
        x = mpz_getlimbn(z, i)

        # Computing modulo 2 ^ hash_bits - 1 means that the bit at
        # position j is really moved to position (j % hash_bits).
        # We need to shift every bit of x left by (limb_bits * i)
        # and then put it in the right position to account for
        # the modulo operation. Store the result in y.
        if limb_bits == hash_bits:
            y = x
        else:
            r = (limb_bits * i) % hash_bits
            y = (x << r) & modulus
            y += (x >> (hash_bits - r)) & modulus
            # Only do this shift if we don't shift more than the size of the
            # type
            if r > 2 * hash_bits - limb_bits:
                y += (x >> (2 * hash_bits - r))
            # At this point, y <= 2 * modulus, so y did not overflow, but we
            # need y <= modulus. We use > instead of >= on the line below
            # because it generates more efficient code.
            if y > modulus:
                y -= modulus

        # Safely compute h = (h + y) % modulus knowing that h < modulus
        # and y <= modulus
        if h < modulus - y:
            h = h + y
        else:
            h = h - (modulus - y)

    # Special case for Python 2
    if limb_bits == hash_bits and h == 0:
        h = -1

    if mpz_sgn(z) < 0:
        return -h
    return h


assert hash_bits <= limb_bits <= 2 * hash_bits

cdef bytes mpz_get_bytes(mpz_srcptr z):
    # NOTE: copied from sage.rings.integer.Integer.str
    cdef char *s
    cdef size_t n
    n = mpz_sizeinbase(z, 10) + 2
    s = <char*>malloc(n)
    mpz_get_str(s, 10, z)
    k = <bytes>s
    free(s)
    return k

cdef bytes mpq_get_bytes(mpq_srcptr z):
    # NOTE: copied from sage.rings.rational.Rational.str
    cdef size_t n
    cdef char*s
    n = mpz_sizeinbase(mpq_numref(z), 10) + \
        mpz_sizeinbase(mpq_denref(z), 10) + 3    
    s = <char *> malloc(n * sizeof(char))
    mpq_get_str(s, 10, z)
    k = <bytes>s
    free(s)
    return k

# The except value is just some random double, it doesn't matter what it is.
cdef double mpz_get_d_nearest(mpz_srcptr x) except? -648555075988944.5:
    cdef mp_bitcnt_t sx = mpz_sizeinbase(x, 2)

    # Easy case: x is exactly representable as double.
    if sx <= 53:
        return mpz_get_d(x)

    cdef int resultsign = mpz_sgn(x)

    # Check for overflow
    if sx > 1024:
        if resultsign < 0:
            return -1.0/0.0
        else:
            return 1.0/0.0

    # General case

    # We should shift x right by this amount in order
    # to have 54 bits remaining.
    cdef mp_bitcnt_t shift = sx - 54

    # Compute q = trunc(x / 2^shift) and let remainder_is_zero be True
    # if and only if no truncation occurred.
    cdef int remainder_is_zero
    remainder_is_zero = mpz_divisible_2exp_p(x, shift)

    sig_on()

    cdef mpz_t q
    mpz_init(q)
    mpz_tdiv_q_2exp(q, x, shift)

    # Convert abs(q) to a 64-bit integer.
    cdef mp_limb_t* q_limbs = (<mpz_ptr>q)._mp_d
    cdef uint64_t q64
    if sizeof(mp_limb_t) >= 8:
        q64 = q_limbs[0]
    else:
        assert sizeof(mp_limb_t) == 4
        q64 = q_limbs[1]
        q64 = (q64 << 32) + q_limbs[0]

    mpz_clear(q)
    sig_off()

    # Round q from 54 to 53 bits of precision.
    if ((q64 & 1) == 0):
        # Round towards zero
        pass
    else:
        if not remainder_is_zero:
            # Remainder is non-zero: round away from zero
            q64 += 1
        else:
            # Halfway case: round to even
            q64 += (q64 & 2) - 1

    # The conversion of q64 to double is *exact*.
    # This is because q64 is even and satisfies 2^53 <= q64 <= 2^54.
    cdef double d = <double>q64
    if resultsign < 0:
        d = -d
    return ldexp(d, shift)



r"""
Polymake matrices

This file contains wrappers for polymake matrices

- Matrix
- SparseMatrix
- IncidenceMatrix

The matrix constructor accepts the following format:

- Matrix(): initialize an empty matrix
- Matrix(values): initialize a matrix from a list of lists
- Matrix(nr, nc): initialize a zero matrix from given sizes
- Matrix(nr, nc, values): initialize a matrix with given dimensions and values
  (either as a plain list or as a list of lists)
"""
###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from cpython.object cimport Py_LT, Py_LE, Py_EQ, Py_NE, Py_GT, Py_GE

from libcpp cimport bool
from libcpp.string cimport string

from libc.stdlib cimport malloc, free
from .cygmp.types cimport mpz_t, mpq_t, mpz_srcptr, mpq_srcptr
from .cygmp.mpz cimport *
from .cygmp.mpq cimport *

from .defs cimport pm_Integer, pm_Rational, pm_Matrix_get, pm_repr_wrap, ostringstream

from .integer cimport Integer
from .rational cimport Rational
from .quadratic_extension cimport QuadraticExtension

#    def python(self):
#        r"""Converts to a list of list of fractions
#
#        >>> import polymake
#
#        >>> c = polymake.cube(3)
#        >>> m = c.DEGREE_ONE_GENERATORS.python()
#        >>> m
#        [[1, -1, -1, -1],
#         [1, -1, -1, 0],
#         [1, -1, -1, 1],
#        ...
#         [1, 1, 1, 0],
#         [1, 1, 1, 1]]
#        >>> type(m), type(m[0]), type(m[0][0])
#        (<type 'list'>, <type 'list'>, <type 'int'>)
#
#        >>> c = polymake.cube(3)
#        >>> m = c.VERTEX_NORMALS.python()
#        >>> m
#        [[Fraction(1, 2), Fraction(1, 2), Fraction(1, 2), Fraction(1, 2)],
#         [Fraction(1, 2), Fraction(-1, 2), Fraction(1, 2), Fraction(1, 2)],
#         [Fraction(1, 2), Fraction(1, 2), Fraction(-1, 2), Fraction(1, 2)],
#        ...
#         [Fraction(1, 2), Fraction(1, 2), Fraction(-1, 2), Fraction(-1, 2)],
#         [Fraction(1, 2), Fraction(-1, 2), Fraction(-1, 2), Fraction(-1, 2)]]
#
#        >>> c = polymake.cube(3)
#        >>> m = c.EDGE_ORIENTATION.python()
#        [[0, 4],
#         [2, 6],
#         [0, 2],
#         ...
#         [2, 3],
#         [6, 7]]
#        >>> type(m), type(m[0]), type(m[0][0])
#        (<type 'list'>, <type 'list'>, <type 'int'>)
#        """
#        cdef Py_ssize_t i, j, nrows, ncols
#        nrows = self.rows()
#        ncols = self.cols()
#        try:
#            return [[self[i,j].python() for j in range(ncols)] for i in range(nrows)]
#        except AttributeError:
#            return [[self[i,j] for j in range(ncols)] for i in range(nrows)]

cdef class PmMatrix(PmObject):
    def __init__(self, *args):
        cdef int nr, nc

        if len(args) == 0:
            # zero matrix
            return

        elif len(args) == 1:
            if isinstance(args[0], PmMatrix):
                # a matrix
                self._set_matrix(args[0])
            else:
                # a list of lists
                nr = len(args[0])
                if not nr:
                    nc = 0
                else:
                    nc = len(args[0][0])
                self.resize(nr, nc)
                if nr and nc:
                    self._set_list_of_lists(args[0])

        elif len(args) == 2:
            # nr, nc
            self.resize(args[0], args[1])

        elif len(args) == 3:
            # nr, nc, data
            if not isinstance(args[2], (tuple, list)):
                raise TypeError
            self.resize(args[0], args[1])
            if not args[0] or not args[1]:
                return
            if isinstance(args[2][0], (tuple, list)):
                self._set_list_of_lists(args[2])
            else:
                self._set_plain_list(args[2])
        else:
            raise ValueError("invalid matrix input")

    cpdef resize(self, int nr, int nc): raise NotImplementedError
    cpdef int rows(self): raise NotImplementedError
    cpdef int cols(self): raise NotImplementedError
    cpdef _set_list_of_lists(self, data): raise NotImplementedError
    cpdef _set_matrix(self, mat): raise NotImplementedError
    cpdef _set_plain_list(self, mat): raise NotImplementedError

cdef class MatrixInt(PmMatrix):
    cpdef int rows(self): return self.pm_obj.rows()
    cpdef int cols(self): return self.pm_obj.cols()
    cpdef resize(self, int nr, int nc): self.pm_obj.resize(nr, nc)

    cpdef _set_list_of_lists(self, data):
        cdef int i, j
        for i,row in enumerate(data):
            for j,elt in enumerate(row):
                pm_Matrix_set(self.pm_obj, i, j, <int> elt)

    cpdef _set_matrix(self, mat):
        if isinstance(mat, MatrixInt):
            self.pm_obj = (<MatrixInt> mat).pm_obj
        else:
            raise NotImplementedError

    cpdef _set_plain_list(self, mat):
        cdef int i,j,k
        k = 0
        for i in range(self.pm_obj.rows()):
            for j in range(self.pm_obj.cols()):
                pm_Matrix_set(self.pm_obj, i, j, <int> (mat[k]))
                k += 1

    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, elt):
        cdef Py_ssize_t nrows, ncols, i,j
        nrows = self.pm_obj.rows()
        ncols = self.pm_obj.cols()
        i,j = elt
        if not (0 <= i < nrows) or not (0 <= j < ncols):
            raise IndexError("matrix index out of range")

        return pm_Matrix_get(self.pm_obj, i, j)

    cpdef _richcmp_(self, other, int op):
        cdef pm_Matrix[int] * left = & (<MatrixInt> self).pm_obj
        cdef pm_Matrix[int] * right = & (<MatrixInt> other).pm_obj

        if op == Py_EQ:
            return left[0] == right[0]
        elif op == Py_NE:
            return left[0] != right[0]
        else:
            raise NotImplementedError

    def sage(self):
        r"""
        Converts to a Sage matrix
        """
        from .sage_conversion import MatrixInt_to_sage
        return MatrixInt_to_sage(self)

    cpdef _add_(self, other):
        cdef MatrixInt ans = MatrixInt.__new__(MatrixInt)
        ans.pm_obj = (<MatrixInt> self).pm_obj + (<MatrixInt> other).pm_obj
        return ans

    cpdef _sub_(self, other):
        cdef MatrixInt ans = MatrixInt.__new__(MatrixInt)
        ans.pm_obj = (<MatrixInt> self).pm_obj - (<MatrixInt> other).pm_obj
        return ans

cdef class MatrixFloat(PmMatrix):
    cpdef int rows(self): return self.pm_obj.rows()
    cpdef int cols(self): return self.pm_obj.cols()
    cpdef resize(self, int nr, int nc): self.pm_obj.resize(nr, nc)

    cpdef _set_list_of_lists(self, data):
        cdef int i, j
        for i,row in enumerate(data):
            for j,elt in enumerate(row):
                pm_Matrix_set(self.pm_obj, i, j, <float> elt)

    cpdef _set_matrix(self, mat):
        if isinstance(mat, MatrixFloat):
            self.pm_obj = (<MatrixFloat> mat).pm_obj
        else:
            raise NotImplementedError

    cpdef _set_plain_list(self, mat):
        cdef int i,j,k
        k = 0
        for i in range(self.pm_obj.rows()):
            for j in range(self.pm_obj.cols()):
                pm_Matrix_set(self.pm_obj, i, j, <float> (mat[k]))
                k += 1

    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, elt):
        cdef Py_ssize_t nrows, ncols, i,j
        nrows = self.pm_obj.rows()
        ncols = self.pm_obj.cols()
        i,j = elt
        if not (0 <= i < nrows) or not (0 <= j < ncols):
            raise IndexError("matrix index out of range")

        return pm_Matrix_get(self.pm_obj, i, j)

    cpdef _richcmp_(self, other, int op):
        cdef pm_Matrix[float] * left = & (<MatrixFloat> self).pm_obj
        cdef pm_Matrix[float] * right = & (<MatrixFloat> other).pm_obj

        if op == Py_EQ:
            return left[0] == right[0]
        elif op == Py_NE:
            return left[0] != right[0]
        else:
            raise NotImplementedError

    def sage(self):
        r"""
        Converts to a Sage matrix
        """
        from .sage_conversion import MatrixFloat_to_sage
        return MatrixFloat_to_sage(self)

cdef class MatrixInteger(PmMatrix):
    cpdef int rows(self): return self.pm_obj.rows()
    cpdef int cols(self): return self.pm_obj.cols()
    cpdef resize(self, int nr, int nc): self.pm_obj.resize(nr, nc)

    cpdef _set_list_of_lists(self, data):
        cdef int i, j
        cdef Integer z
        for i,row in enumerate(data):
            for j,elt in enumerate(row):
                # TODO: we should not create a new integer each time but rather
                # reset the value
                z = Integer(elt)
                pm_Matrix_set(self.pm_obj, i, j, z.pm_obj)

    cpdef _set_matrix(self, mat):
        if isinstance(mat, MatrixFloat):
            self.pm_obj = (<MatrixFloat> mat).pm_obj
        else:
            raise NotImplementedError

    cpdef _set_plain_list(self, mat):
        cdef int i,j,k
        cdef Integer z
        k = 0
        for i in range(self.pm_obj.rows()):
            for j in range(self.pm_obj.cols()):
                z = Integer(mat[k])
                pm_Matrix_set(self.pm_obj, i, j, z.pm_obj)
                k += 1

    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, elt):
        cdef Py_ssize_t nrows, ncols, i,j
        nrows = self.pm_obj.rows()
        ncols = self.pm_obj.cols()
        i,j = elt
        if not (0 <= i < nrows) or not (0 <= j < ncols):
            raise IndexError("matrix index out of range")

        cdef Integer ans = Integer.__new__(Integer)
        ans.pm_obj.set_mpz_srcptr(pm_Matrix_get(self.pm_obj, i, j).get_rep())
        return ans

    cpdef _richcmp_(self, other, int op):
        cdef pm_Matrix[pm_Integer] * left = & (<MatrixInteger> self).pm_obj
        cdef pm_Matrix[pm_Integer] * right = & (<MatrixInteger> other).pm_obj

        if op == Py_EQ:
            return left[0] == right[0]
        elif op == Py_NE:
            return left[0] != right[0]
        else:
            raise NotImplementedError

    def sage(self):
        r"""
        Converts to a Sage integer matrix
        """
        from .sage_conversion import MatrixInteger_to_sage
        return MatrixInteger_to_sage(self)

cdef class MatrixRational(PmMatrix):
    cpdef int rows(self): return self.pm_obj.rows()
    cpdef int cols(self): return self.pm_obj.cols()
    cpdef resize(self, int nr, int nc): self.pm_obj.resize(nr, nc)

    cpdef _set_list_of_lists(self, data):
        cdef int i, j
        cdef Rational q
        for i,row in enumerate(data):
            for j,elt in enumerate(row):
                # TODO: we should not create a new rational each time
                # but rather reset the value
                q = Rational(elt)
                pm_Matrix_set(self.pm_obj, i, j, q.pm_obj)

    cpdef _set_matrix(self, mat):
        if isinstance(mat, MatrixFloat):
            self.pm_obj = (<MatrixFloat> mat).pm_obj
        else:
            raise NotImplementedError

    cpdef _set_plain_list(self, mat):
        cdef int i,j,k
        cdef Rational q
        k = 0
        for i in range(self.pm_obj.rows()):
            for j in range(self.pm_obj.cols()):
                # TODO: we should not create a new rational each time
                # but rather reset the value
                q = Rational(mat[k])
                pm_Matrix_set(self.pm_obj, i, j, q.pm_obj)
                k += 1

    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, elt):
        cdef Py_ssize_t nrows, ncols, i, j
        nrows = self.pm_obj.rows()
        ncols = self.pm_obj.cols()
        i,j = elt
        if not (0 <= i < nrows) or not (0 <= j < ncols):
            raise IndexError("matrix index out of range")

        cdef Rational ans = Rational.__new__(Rational)
        cdef mpq_srcptr q = pm_Matrix_get(self.pm_obj, i, j).get_rep()
        ans.pm_obj.set_mpq_srcptr(q)
        return ans

    cpdef _richcmp_(self, other, int op):
        cdef pm_Matrix[pm_Rational] * left = & (<MatrixRational> self).pm_obj
        cdef pm_Matrix[pm_Rational] * right = & (<MatrixRational> other).pm_obj

        if op == Py_EQ:
            return left[0] == right[0]
        elif op == Py_NE:
            return left[0] != right[0]
        else:
            raise NotImplementedError

    def sage(self):
        r"""
        Converts into a Sage matrix
        """
        from .sage_conversion import MatrixRational_to_sage
        return MatrixRational_to_sage(self)

cdef class MatrixQuadraticExtension(PmMatrix):
    cpdef int rows(self): return self.pm_obj.rows()
    cpdef int cols(self): return self.pm_obj.cols()
    cpdef resize(self, int nr, int nc): self.pm_obj.resize(nr, nc)

    cpdef _set_list_of_lists(self, data):
        cdef int i, j
        cdef QuadraticExtension qe
        for i,row in enumerate(data):
            for j,elt in enumerate(row):
                # TODO: we should not create a new quadratic extension
                # integer each time but rather reset the value
                qe = QuadraticExtension(elt)
                pm_Matrix_set(self.pm_obj, i, j, qe.pm_obj)

    cpdef _set_matrix(self, mat):
        if isinstance(mat, MatrixFloat):
            self.pm_obj = (<MatrixFloat> mat).pm_obj
        else:
            raise NotImplementedError

    cpdef _set_plain_list(self, mat):
        cdef int i,j,k
        cdef QuadraticExtension qe
        k = 0
        for i in range(self.pm_obj.rows()):
            for j in range(self.pm_obj.cols()):
                # TODO: we should not create a new quadratic extension
                # integer each time but rather reset the value
                qe = QuadraticExtension(mat[k])
                pm_Matrix_set(self.pm_obj, i, j, qe.pm_obj)
                k += 1

    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, elt):
        cdef Py_ssize_t nrows, ncols, i, j
        nrows = self.pm_obj.rows()
        ncols = self.pm_obj.cols()
        i,j = elt
        if not (0 <= i < nrows) or not (0 <= j < ncols):
            raise IndexError("matrix index out of range")

        cdef QuadraticExtension ans = QuadraticExtension.__new__(QuadraticExtension)
        # TODO: this is a lot of useless copies
        cdef pm_Rational qa = pm_Matrix_get(self.pm_obj, i, j).a()
        cdef pm_Rational qb = pm_Matrix_get(self.pm_obj, i, j).b()
        cdef pm_Rational qr = pm_Matrix_get(self.pm_obj, i, j).r()
        ans.pm_obj = pm_QuadraticExtension(qa, qb, qr)
        return ans

    def sage(self, field=None):
        from .sage_conversion import MatrixQuadraticExtension_to_sage
        return MatrixQuadraticExtension_to_sage(self, field)



from cpython.object cimport PyObject, PyTypeObject
from libc.string cimport strncmp

import numbers

from .pm_object cimport PmObject

# TODO: we should try to not import inside!

cpdef as_pm_object(x):
    # fast type checking
    if isinstance(x, (int, long)):
        from .integer import Integer
        return Integer(x)

    # slower type checking
    if isinstance(x, numbers.Integral):
        from .integer import Integer
        return Integer(x)

    if isinstance(x, numbers.Rational):
        from .rational import Rational
        return Rational(x)

    if is_sage_type(type(x)):
        from .sage_conversion import (QuadraticExtension_from_sage,
                Polytope_from_sage,
                VectorInteger_from_sage, MatrixInteger_from_sage,
                VectorRational_from_sage, MatrixRational_from_sage,
                VectorQuadraticExtension_from_sage, MatrixQuadraticExtension_from_sage)
        from sage.all import ZZ, QQ
        from sage.structure.element import Matrix
        from sage.geometry.polyhedron.base import Polyhedron_base
        from sage.modules.free_module_element import FreeModuleElement
        from sage.rings.number_field.number_field import NumberField_quadratic
        from sage.rings.number_field.number_field_element_quadratic import NumberFieldElement_quadratic

        if isinstance(x, NumberFieldElement_quadratic):
            return QuadraticExtension_from_sage(x)
        elif isinstance(x, FreeModuleElement):
            if x.base_ring() is ZZ:
                if x.is_dense():
                    return VectorInteger_from_sage(x)
                else:
                    from .vector import VectorInteger
                    return VectorInteger(x)
            elif x.base_ring() is QQ:
                if x.is_dense():
                    return VectorRational_from_sage(x)
                else:
                    from .rational import VectorRational
                    return VectorRational(x)
            elif isinstance(x.base_ring(), NumberField_quadratic):
                return VectorQuadraticExtension_from_sage(x)
            else:
                raise NotImplementedError("no equivalent polymake vector")
        elif isinstance(x, Matrix):
            if x.base_ring() is ZZ:
                return MatrixInteger_from_sage(x)
            elif x.base_ring() is QQ:
                return MatrixRational_from_sage(x)
            elif isinstance(x.base_ring(), NumberField_quadratic):
                return MatrixQuadraticExtension_from_sage(x)
            else:
                raise NotImplementedError("no equivalent polymake matrix")
        elif isinstance(x, Polyhedron_base):
            return Polytope_from_sage(x)
        else:
            raise NotImplementedError("no equivalent polymake object")

    raise NotImplementedError("no equivalent polymake object")

# TODO: this is very slow!
cpdef bint is_sage_type(t):
    return isinstance(t, type) and t.__module__.startswith('sage.')

cpdef canonical_coercion(x, y):
    if type(x) is type(y):
        return (x,y)

    if not isinstance(x, PmObject):
        x = as_pm_object(x)
    if not isinstance(y, PmObject):
        y = as_pm_object(y)

    if type(x) is type(y):
        return (x, y)

    from .integer import Integer
    from .rational import Rational
    from .quadratic_extension import QuadraticExtension
    from .vector import VectorInteger, VectorRational

    if type(x) is Integer and type(y) is Rational:
        return (Rational(x), y)
    if type(x) is Rational and type(y) is Integer:
        return (x, Rational(y))
    if type(x) in [Integer, Rational] and type(y) is QuadraticExtension:
        return (QuadraticExtension(x), y)
    if type(x) is QuadraticExtension and type(y) in [Integer, Rational]:
        return (x, QuadraticExtension(y))
    if type(x) is VectorInteger and type(y) is VectorRational:
        return (VectorRational(x), y)
    if type(x) is VectorRational and type(y) is VectorInteger:
        return (x, VectorRational(y))

    raise ValueError("no coercion found for x={} (of type {}) and y={} (of type {})".format(x, type(x), y, type(y)))



from .coercion cimport canonical_coercion

cdef inline int classify_element(left, right):
    if type(left) is type(right):
        return 0o37
    if not isinstance(right, PmObject):
        return 0o01
    if not isinstance(left, PmObject):
        return 0o02
    return 0o07

cdef inline bint ARE_SAME_PMOBJECTS(int cl):
    return cl & 0o20
cdef inline bint BOTH_ARE_PMOBJECTS(int cl):
    return cl & 0o04

cdef class PmObject(object):
    cpdef _add_(self, other): raise NotImplementedError
    cpdef _sub_(self, other): raise NotImplementedError
    cpdef _mul_(self, other): raise NotImplementedError
    cpdef _div_(self, other): raise NotImplementedError
    cpdef _floordiv_(self, other): raise NotImplementedError
    cpdef _richcmp_(self, other, int op): raise NotImplementedError

    def __add__(self, other):
        cdef int cl = classify_element(self, other)
        if not ARE_SAME_PMOBJECTS(cl):
            self, other = canonical_coercion(self, other)
        return (<PmObject> self)._add_(other)

    def __sub__(self, other):
        cdef int cl = classify_element(self, other)
        if not ARE_SAME_PMOBJECTS(cl):
            self, other = canonical_coercion(self, other)
        return (<PmObject> self)._sub_(other)

    def __mul__(self, other):
        cdef int cl = classify_element(self, other)
        if not ARE_SAME_PMOBJECTS(cl):
            self, other = canonical_coercion(self, other)
        return (<PmObject> self)._mul_(other)

    def __truediv__(self, other):
        cdef int cl = classify_element(self, other)
        if not ARE_SAME_PMOBJECTS(cl):
            self, other = canonical_coercion(self, other)
        if not other:
            raise ZeroDivisionError
        return (<PmObject> self)._div_(other)

    def __div__(self, other):
        cdef int cl = classify_element(self, other)
        if not ARE_SAME_PMOBJECTS(cl):
            self, other = canonical_coercion(self, other)
        if not other:
            raise ZeroDivisionError
        return (<PmObject> self)._div_(other)

    def __floordiv__(self, other):
        cdef int cl = classify_element(self, other)
        if not ARE_SAME_PMOBJECTS(cl):
            self, other = canonical_coercion(self, other)
        if not other:
            raise ZeroDivisionError
        return (<PmObject> self)._floordiv_(other)

    def __richcmp__(self, other, op):
        cdef int cl = classify_element(self, other)
        if not ARE_SAME_PMOBJECTS(cl):
            self, other = canonical_coercion(self, other)
        return (<PmObject> self)._richcmp_(other, op)



"""
Various functions to deal with conversion mpz <-> Python int/long
"""
from .types cimport *

cdef bytes mpz_get_bytes(mpz_srcptr)
cdef bytes mpq_get_bytes(mpq_srcptr)
cdef mpz_get_pylong(mpz_srcptr z)
cdef mpz_get_pyintlong(mpz_srcptr z)
cdef int mpz_set_pylong(mpz_ptr z, L) except -1
cdef Py_hash_t mpz_pythonhash(mpz_srcptr z)
cdef double mpz_get_d_nearest(mpz_srcptr x) except? -648555075988944.5



###############################################################################
#       Copyright (C) 2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr> 
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from cpython.object cimport Py_LT, Py_LE, Py_EQ, Py_NE, Py_GT, Py_GE

from libcpp cimport bool
from libcpp.string cimport string

from .defs cimport pm_QuadraticExtension, pm_repr_wrap
from .integer cimport Integer
from .rational cimport Rational

import numbers

# this is a bug in Cython!
def _NOT_TO_BE_USED_():
    raise ValueError
def _NOT_TO_BE_USED_():
    raise TypeError

cpdef Rational to_rat(a):
    if type(a) is Rational:
        return a
    elif a is None:
        return Rational(0)
    else:
        return Rational(a)

cdef class QuadraticExtension(PmObject):
    r"""
    Element of a quadratic extension of the rational numbers

    This class implements numbers of the form a + b sqrt(r).
    """
    def __init__(self, a=None, b=None, r=None):
        if a is None and b is None:
            return

        if b is None and r is None:
            if type(a) is QuadraticExtension:
                self.pm_obj = (<QuadraticExtension> a).pm_obj
                return
            elif isinstance(a, (list, tuple)):
                if len(a) == 3:
                    a, b, r = a
                else:
                    raise ValueError

        cdef Rational ra = to_rat(a)
        if b is None or r is None:
            self.pm_obj = pm_QuadraticExtension(ra.pm_obj)
            return

        cdef Rational rb = to_rat(b)
        cdef Rational rr = to_rat(r)

        self.pm_obj = pm_QuadraticExtension(ra.pm_obj, rb.pm_obj, rr.pm_obj)

    def __nonzero__(self):
        return not pm_is_zero(self.pm_obj)

    def a(self):
        cdef Rational a = Rational.__new__(Rational)
        a.pm_obj.set_mpq_srcptr(self.pm_obj.a().get_rep())
        return a

    def b(self):
        cdef Rational b = Rational.__new__(Rational)
        b.pm_obj.set_mpq_srcptr(self.pm_obj.b().get_rep())
        return b

    def r(self):
        cdef Rational r = Rational.__new__(Rational)
        r.pm_obj.set_mpq_srcptr(self.pm_obj.r().get_rep())
        return r

    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    cpdef _add_(self, other):
        cdef QuadraticExtension ans = QuadraticExtension.__new__(QuadraticExtension)
        ans.pm_obj = (<QuadraticExtension> self).pm_obj + \
                     (<QuadraticExtension> other).pm_obj
        return ans

    cpdef _sub_(self, other):
        cdef QuadraticExtension ans = QuadraticExtension.__new__(QuadraticExtension)
        ans.pm_obj = (<QuadraticExtension> self).pm_obj - \
                     (<QuadraticExtension> other).pm_obj
        return ans

    cpdef _mul_(self, other):
        cdef QuadraticExtension ans = QuadraticExtension.__new__(QuadraticExtension)
        ans.pm_obj = (<QuadraticExtension> self).pm_obj * \
                     (<QuadraticExtension> other).pm_obj
        return ans

    cpdef _div_(self, other):
        cdef QuadraticExtension ans = QuadraticExtension.__new__(QuadraticExtension)
        ans.pm_obj = (<QuadraticExtension> self).pm_obj / \
                     (<QuadraticExtension> other).pm_obj
        return ans

    cpdef _richcmp_(self, other, int op):
        cdef QuadraticExtension a = <QuadraticExtension> self
        cdef QuadraticExtension b = <QuadraticExtension> other

        if a.pm_obj == b.pm_obj:
            return op in [Py_EQ, Py_LE, Py_GE]
        elif op == Py_NE:
            return True
        elif op == Py_EQ:
            return False

        if a.pm_obj < b.pm_obj:
            return op in [Py_LE, Py_LT]
        else:
            return op in [Py_GE, Py_GT]

    def __neg__(self):
        cdef QuadraticExtension ans = QuadraticExtension.__new__(QuadraticExtension)
        ans.pm_obj = - self.pm_obj
        return ans

    def sage(self, field=None):
        r"""
        Convert this quadratic extension element to SageMath
        """
        from .sage_conversion import QuadraticExtension_to_sage
        return QuadraticExtension_to_sage(self, field)

import numbers
numbers.Real.register(QuadraticExtension)



# This file was autogenerated! Do not modify!
# power_set interfacing PowerSet.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class PowerSetInt(PmObject):
    cdef pm_PowerSet[int] pm_obj



#from polymake.main import pm_set_application, functions_in_current_application, arguments

class PolymakeFunctionWrapper(object):
    def __init__(self, app, name):
        if isinstance(app, str):
            app = app.encode('ascii')
        if isinstance(name, str):
            name = name.encode('ascii')
        self._app = app
        self._name = name
        self._args = arguments(self._name)
    def __repr__(self):
        s = "Function {}::{}\n".format(self._app.decode(), self._name.decode())
        args = []
        for arg in self._args:
            args.append("  " + ", ".join(v.decode() + ":" + typ.decode() for (v,typ) in arg))
        return s + "\n".join(args)

    def __call__(self, *args, **kwds):
        raise ValueError

class PolymakeApplication(object):
    def __init__(self, name):
        self._name = name.encode('ascii')
        self._populate_functions()
    def __repr__(self):
        print("Application {}".format(self._name))
    def _populate_functions(self):
        # TODO: this is broken for now...
        return
        pm_set_application(self._name)
        for f in functions_in_current_application():
            try:
                self.__setattr__( f, PolymakeFunctionWrapper(self._name, f))
            except RuntimeError as err:
                print("impossible for now to set {}".format(f))

common   = PolymakeApplication("common")
fulton   = PolymakeApplication("fulton")
group    = PolymakeApplication("group")
matroid  = PolymakeApplication("matroid")
topaz    = PolymakeApplication("topaz")
fan      = PolymakeApplication("fan")
graph    = PolymakeApplication("graph")
ideal    = PolymakeApplication("ideal")
polytope = PolymakeApplication("polytope")
tropical = PolymakeApplication("tropical")




from .types cimport *

cdef extern from "gmp.h":

    ### Rational Functions ###

    void mpq_canonicalize (mpq_t op)

    # Initialization and Assignment Functions
    void mpq_init (mpq_t dest_rational)
    void mpq_clear (mpq_t rational_number)
    void mpq_set (mpq_t rop, mpq_t op)
    void mpq_set_z (mpq_t rop, mpz_t op)
    void mpq_set_ui (mpq_t rop, unsigned long int op1, unsigned long int op2)
    void mpq_set_si (mpq_t rop, signed long int op1, unsigned long int op2)
    int mpq_set_str (mpq_t rop, char *str, int base)
    void mpq_swap (mpq_t rop1, mpq_t rop2)

    # Conversion Functions
    double mpq_get_d (mpq_t op)
    void mpq_set_d (mpq_t rop, double op)
    void mpq_set_f (mpq_t rop, mpf_t op)
    char * mpq_get_str (char *str, int base, mpq_t op)

    # Arithmetic Functions
    void mpq_add (mpq_t sum, mpq_t addend1, mpq_t addend2)
    void mpq_sub (mpq_t difference, mpq_t minuend, mpq_t subtrahend)
    void mpq_mul (mpq_t product, mpq_t multiplier, mpq_t multiplicand)
    void mpq_mul_2exp (mpq_t rop, mpq_t op1, unsigned long int op2)
    void mpq_div (mpq_t quotient, mpq_t dividend, mpq_t divisor)
    void mpq_div_2exp (mpq_t rop, mpq_t op1, unsigned long int op2)
    void mpq_neg (mpq_t negated_operand, mpq_t operand)
    void mpq_abs (mpq_t rop, mpq_t op)
    void mpq_inv (mpq_t inverted_number, mpq_t number)

    # Comparison Functions
    int mpq_cmp (mpq_t op1, mpq_t op2)
    int mpq_cmp_ui (mpq_t op1, unsigned long int num2, unsigned long int den2)
    int mpq_cmp_si (mpq_t op1, long int num2, unsigned long int den2)
    int mpq_sgn (mpq_t op)
    int mpq_equal (mpq_t op1, mpq_t op2)

    # Applying Integer Functions to Rationals
    mpz_t mpq_numref (mpq_t op)
    mpz_t mpq_denref (mpq_t op)
    void mpq_get_num (mpz_t numerator, mpq_t rational)
    void mpq_get_den (mpz_t denominator, mpq_t rational)
    void mpq_set_num (mpq_t rational, mpz_t numerator)
    void mpq_set_den (mpq_t rational, mpz_t denominator)

    # Input and Output Functions
    # size_t mpq_out_str (file *stream, int base, mpq_t op)
    # size_t mpq_inp_str (mpq_t rop, file *stream, int base)




r"""
Handlers for polymake objects

This file organizes the conversion of C++ polymake objects
to the actual wrappers of the pypolymake library. Most of the code
is actually generated automatically at build time in "auto_handlers.pxi"
and "auto_mappings.pxi".
"""
###############################################################################
#       Copyright (C) 2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from libcpp cimport bool
from libcpp.string cimport string

from cysignals.signals cimport sig_on, sig_off, sig_str

from .defs cimport *
from .perl_object cimport *

from .array cimport *
from .incidence_matrix cimport *
from .integer cimport *
from .map cimport *
from .matrix cimport *
from .polynomial cimport *
from .power_set cimport *
from .quadratic_extension cimport *
from .rational cimport *
from .rational_function cimport *
from .set cimport *
from .sparse_matrix cimport *
from .vector cimport *

cdef extern from "wrap.h" namespace "polymake":
    void pm_give_PerlObject "GIVE" (pm_PerlObject, pm_PerlObject*, string)
    void pm_call_method_PerlObject "CALL_METHOD" (pm_PerlObject, pm_PerlObject*, string)

    void pm_give_int "GIVE" (int, pm_PerlObject*, string) except +
    void pm_call_method_int "CALL_METHOD" (int, pm_PerlObject*, string) except +

    void pm_give_float "GIVE" (float, pm_PerlObject*, string) except +
    void pm_call_method_float "CALL_METHOD" (float, pm_PerlObject*, string) except +


def give_generic(PerlObject perl_object, bytes prop):
    cdef pm_PerlObject pm_ans
    cdef string cprop = prop
    sig_str("error in polymake while accessing property")
    pm_give_PerlObject(pm_ans, perl_object.pm_obj, prop)
    sig_off()
    if not pm_ans.valid():
        raise ValueError("invalid property {}".format(prop))
    return wrap_perl_object(pm_ans)

def call_method_generic(PerlObject perl_object, bytes prop):
    cdef pm_PerlObject pm_ans
    cdef string cprop = prop
    sig_str("error in polymke while accessing method")
    pm_call_method_PerlObject(pm_ans, perl_object.pm_obj, prop)
    sig_off()
    if not pm_ans.valid():
        raise ValueError("invalid property {}".format(prop))
    return wrap_perl_object(pm_ans)

def give_bool(PerlObject perl_object, bytes prop):
    return give_int(perl_object, prop)

def call_method_bool(PerlObject perl_object, bytes prop):
    return call_method_int(perl_object, prop)

def give_int(PerlObject perl_object, bytes prop):
    cdef int ans
    pm_give_int(ans, perl_object.pm_obj, prop)
    return ans
def call_method_int(PerlObject perl_object, bytes prop):
    cdef int ans
    pm_call_method_int(ans, perl_object.pm_obj, prop)
    return ans

def give_float(PerlObject perl_object, bytes prop):
    cdef float ans
    pm_give_float(ans, perl_object.pm_obj, prop)
    return ans
def call_method_float(PerlObject perl_object, bytes prop):
    cdef float ans
    pm_call_method_float(ans, perl_object.pm_obj, prop)
    return ans

include "auto_handlers.pxi"

cdef dict property_handlers = {
    b"Bool"          : give_bool,
    b"Int"           : give_int,
    b"Float"         : give_float,
}
cdef dict method_handlers = {
    b"Bool"          : call_method_bool,
    b"Int"           : call_method_int,
    b"Float"         : call_method_float
}
include "auto_mappings.pxi"
property_handlers.update(auto_property_handlers)
method_handlers.update(auto_method_handlers)

cpdef bint is_small_type(bytes typ):
    return (typ == b"Integer" or
            typ == b"Rational" or
            typ.startswith(b"Array<") or
            typ.startswith(b"Map<") or
            typ.startswith(b"Matrix<") or
            typ.startswith(b"PowerSet<") or
            typ.startswith(b"RationalFunction<") or
            typ.startswith(b"Set<") or
            typ.startswith(b"SparseMatrix<") or
            typ.startswith(b"SparseVector<") or
            typ.startswith(b"UniPolynomial<") or
            typ.startswith(b"Vector<"))

cpdef get_property_handler(bytes pm_type):
    global property_handlers
    try:
        return property_handlers[pm_type]
    except KeyError:
        if is_small_type(pm_type):
            raise NotImplementedError("pypolymake does not handle {} polymake type".format(pm_type))
        return give_generic

cpdef get_method_handler(bytes pm_type):
    global method_handlers
    cdef bytes typ
    try:
        return method_handlers[pm_type]
    except KeyError:
        if is_small_type(pm_type):
            raise NotImplementedError("pypolymake does not handle {} polymake type".format(pm_type))
        return call_method_generic




# This file was autogenerated! Do not modify!
# sparse_matrix interfacing SparseMatrix.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class SparseMatrixInt(PmObject):
    cdef pm_SparseMatrix[int] pm_obj

cdef class SparseMatrixRational(PmObject):
    cdef pm_SparseMatrix[pm_Rational] pm_obj



# distutils: libraries = gmp m

from libc.stdio cimport FILE

cdef extern from "gmp.h":
    # GMP's configuration of how many bits are stuffed into a limb
    cdef unsigned int GMP_LIMB_BITS
    cdef int mp_bits_per_limb

    # Underlying typedefs
    ctypedef unsigned long mp_limb_t
    ctypedef long mp_limb_signed_t
    ctypedef unsigned long mp_bitcnt_t
    ctypedef long mp_size_t
    ctypedef long mp_exp_t

    ctypedef mp_limb_t* mp_ptr
    ctypedef mp_limb_t* mp_srcptr

    # This internal structure is not guaranteed to stay the same with
    # future releases of GMP or MPIR.
    ctypedef struct __mpz_struct:
        int _mp_alloc
        int _mp_size
        mp_ptr _mp_d

    ctypedef struct __mpq_struct:
        __mpz_struct _mp_num
        __mpz_struct _mp_den

    ctypedef struct __mpf_struct:
        int _mp_prec
        int _mp_size
        mp_exp_t _mp_exp
        mp_limb_t *_mp_d

    ctypedef struct __gmp_randstate_struct:
        pass

    # User facing types
    ctypedef __mpz_struct mpz_t[1]
    ctypedef __mpq_struct mpq_t[1]
    ctypedef __mpf_struct mpf_t[1]
    ctypedef __gmp_randstate_struct gmp_randstate_t[1]

    ctypedef __mpz_struct *mpz_ptr
    ctypedef __mpq_struct *mpq_ptr
    ctypedef __mpf_struct *mpf_ptr
    ctypedef __mpz_struct *mpz_srcptr
    ctypedef __mpq_struct *mpq_srcptr
    ctypedef __mpf_struct *mpf_srcptr

    # Cython doesn't want to take the address of an mpz_t
    cdef mpz_t* address_of_mpz "&"(mpz_t x)



import unittest
import random

# NOTE: apparently this has to be done *before* importing polymake.
# Otherwise it produces segfault...
try:
    import sage.all
    import sage
except ImportError:
    sage = None

import polymake

try:
    import gmpy2
except ImportError:
    gmpy2 = None

class TestVector(unittest.TestCase):
    vecs = [polymake.VectorInteger, polymake.VectorRational]

    def test_init(self):
        polymake.VectorInteger([])
        polymake.VectorInteger([0,1,3])
        polymake.VectorRational([0,-3])

        polymake.VectorRational([(2,1),(-1,3)])
        polymake.VectorRational([2,(-1,3)])
        polymake.VectorRational([(2,5),-1])

    def test_cmp(self):
        for t in self.vecs:
            self.assertTrue(t() == t())
            self.assertFalse(t() != t())
            self.assertEqual(t([0,1]), t([0,1]))

            self.assertFalse(t() == t([0]))
            self.assertTrue(t() != t([0]))
            self.assertNotEqual(t([0]), t())

            self.assertNotEqual(t([0]), t([1]))
            self.assertNotEqual(t([1]), t([0]))

        for t1 in self.vecs:
            for t2 in self.vecs:
                self.assertEqual(t1(), t2())
                self.assertEqual(t1([0,0]), t2([0,0]))
                self.assertNotEqual(t1(), t2([0]))
                self.assertNotEqual(t1([0]), t2())

    def test_neg(self): 
        for t in self.vecs:
            self.assertEqual(-t([1,2]), t([-1,-2]))

    def test_binop(self):
        for t in self.vecs:
            self.assertEqual(t([1]) + t([3]), t([4]))

        for t1 in self.vecs:
            for t2 in self.vecs:
                for t3 in self.vecs:
                    self.assertEqual(t1([1]) + t2([3]), t3([4]))




r"""
Polymake Python library

The polymake python library provides Python bindings to polymake.
"""
from __future__ import absolute_import

from .integer import Integer
from .rational import Rational
from .quadratic_extension import QuadraticExtension

from .vector import VectorInteger, VectorRational

from .functions import *
from .big_object import Polytope

from .coercion import as_pm_object as polymake



###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016      Vincent Delecroix <vincent.delecroix@labri.fr>
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from .perl_object import call_polymake_function

def associahedron(d):
    r"""Produce a d-dimensional associahedron (or Stasheff polytope)

    >>> import polymake
    >>> a = polymake.associahedron(3)
    >>> a.N_VERTICES
    14
    """
    return call_polymake_function(b"polytope", b"associahedron", d)

def birkhoff(n, even=False):
    """Constructs the Birkhoff polytope of dimension `n^2`.

    It is the polytope of `n \\times n` stochastic matrices (encoded as `n^2`
    row vectors), thus matrices with non-negative entries whose row and column
    entries sum up to one. Its vertices are the permutation matrices.

    Keyword arguments:

    n -- integer
    even -- boolean (default to False)

    >>> import polymake
    >>> b = polymake.birkhoff(3)
    >>> b.N_VERTICES
    6
    """
    return call_polymake_function(b"polytope", b"birkhoff", n, even)

def cross(d):
    r"""Produce a d-dimensional cross polytope.

    Regular polytope corresponding to the Coxeter group of type B<sub>d-1</sub>
    = C<sub>d-1</sub>.
    """
    return call_polymake_function(b"polytope", b"cross", d)

def cube(d, x_up=1, x_low=-1):
    r"""Produce a d-dimensional cube

    Regular polytope corresponding to the Coxeter group of type
    `B^{d-1} = C^{d-1}`.  The bounding hyperplanes are `x_i <= x_{up}` and
    `x_i >= x_{low}`.

    Keyword arguments:

    d -- dimension
    x_up -- upper bound in each dimension
    x_low -- lower bound in each dimension

    >>> import polymake
    >>> polymake.cube(2).VERTICES
    1 -1 -1
    1 1 -1
    1 -1 1
    1 1 1
    >>> polymake.cube(2,2,0).VERTICES
    1 0 0
    1 2 0
    1 0 2
    1 2 2
    """
    return call_polymake_function(b"polytope", b"cube", d, x_up, x_low)

def cuboctahedron():
    r"""Create cuboctahedron.  An Archimedean solid.

    >>> import polymake
    >>> polymake.cuboctahedron().VERTICES
    1 1  1  0
    1 1  0  1
    1 0  1  1
    1 1  0  -1
    1 0  1  -1
    1 1  -1 0
    1 0  -1 1
    1 -1 1  0
    1 -1 0  1
    1 0  -1 -1
    1 -1 0  -1
    1 -1 -1 0
    """
    return call_polymake_function(b"polytope", b"cuboctahedron")

def cycle_graph(n):
    r"""Constructs a cycle graph on n nodes.

    >>> import polymake
    >>> g = polymake.cycle_graph(7)
    >>> g
    Graph<Undirected>
    >>> g.N_NODES
    7
    >>> g.N_EDGES
    7
    """
    return call_polymake_function(b"polytope", b"cycle_graph", n)

def cyclic(d, n):
    r"""Produce a d-dimensional cyclic polytope with n points.

    Prototypical example of a neighborly polytope. Combinatorics completely
    known due to Gale's evenness criterion. Coordinates are chosen on the
    (spherical) moment curve at integer steps from start, or 0 if unspecified.
    If spherical is true the vertices lie on the sphere with center
    (1/2,0,...,0) and radius 1/2. In this case (the necessarily positive)
    parameter start defaults to 1.

    >>> import polymake
    >>> polymake.cyclic(2, 4).VERTICES
    1 0 0
    1 1 1
    1 2 4
    1 3 9
    >>> polymake.cyclic(3, 3)
    Traceback (most recent call last):
    ...
    RuntimeError: cyclic: d >= 2 and n > d required
    <BLANKLINE>
    """
    return call_polymake_function(b"polytope", b"cyclic", d, n)

def cyclic_caratheodory(d, n):
    r"""Produce a d-dimensional cyclic polytope with n points.

    Prototypical example of a neighborly polytope. Combinatorics completely known
    due to Gale's evenness criterion. Coordinates are chosen on the trigonometric
    moment curve. For cyclic polytopes from other curves, see :func:`cyclic`.
    """
    return call_polymake_function(b"polytope", b"cyclic_caratheodory", d, n)

def delpezzo(d, scale=1):
    r"""
    Produce a d-dimensional del-Pezzo polytope

    It is the convex hull of the cross polytope together with the all-ones and
    minus all-ones vector. All coordinates are +/- scale or 0.

    Keyword arguments:

    d -- the dimension
    scale -- the absolute value of each non-zero vertex coordinate.

    >>> import polymake
    >>> polymake.delpezzo(2).VERTICES
    1 1  0
    1 0  1
    1 -1 0
    1 0  -1
    1 1 1
    1 -1 -1
    """
    return call_polymake_function(b"polytope", b"delpezzo", d, scale)

def dodecahedron():
    r"""
    Create exact regular dodecahedron in Q(sqrt{5}).  A Platonic solid.

    >>> import polymake
    >>> dod = polymake.dodecahedron()
    >>> dod
    Polytope<QuadraticExtension<Rational>>
    """
    return call_polymake_function(b"polytope", b"dodecahedron")

def dwarfed_cube(d):
    r"""Produce a d-dimensional dwarfed cube.

    Keyword arguments:

    d -- the dimension

    >>> import polymake
    >>> polymake.dwarfed_cube(2).N_VERTICES
    5
    """
    return call_polymake_function(b"polytope", b"dwarfed_cube", d)

def icosahedron():
    r"""
    Create exact regular icosahedron in Q(sqrt{5}).  A Platonic solid.

    >>> import polymake
    >>> ico = polymake.icosahedron()
    >>> ico
    Polytope<QuadraticExtension<Rational>>
    """
    return call_polymake_function(b"polytope", b"icosahedron")

def johnson_graph(n, k):
    r"""Create the Johnson graph on parameters (n,k).

    It has one node for each set in \({[n]}\choose{k}\),
    and an edge between two nodes iff the intersection of the corresponding
    subsets is of size k-1.

    >>> import polymake
    >>> g = polymake.johnson_graph(5, 3)
    >>> g
    Graph<Undirected>
    >>> g.N_NODES
    10
    >>> g.N_EDGES
    30
    """
    return call_polymake_function(b"polytope", b"johnson_graph", n, k)

def kneser_graph(n, k):
    r"""Create the Kneser graph on parameters (n,k).

    It has one node for each set in \({[n]}\choose{k}\),
    and an edge between two nodes iff the corresponding subsets are
    disjoint.

    >>> import polymake
    >>> g = polymake.kneser_graph(6, 3)
    >>> g
    Graph<Undirected>
    >>> g.N_NODES
    20
    >>> g.N_EDGES
    10
    """
    return call_polymake_function(b"polytope", b"kneser_graph", n, k)

def neighborly_cubical(d, n):
    r""" Produce the combinatorial description of a neighborly cubical polytope.

    Keyword arguments:

    d -- dimension of the polytope
    n -- dimension of the equivalent cube

   The facets are labelled in oriented matroid notation as in the cubical Gale
    evenness criterion.  See Joswig and Ziegler, Discr. Comput. Geom.
    24:315-344 (2000).
    """
    return call_polymake_function(b"polytope", b"neighborly_cubical", d, n)


def rand_sphere(dim, npoints):
    r"""Produce a d-dimensional polytope with n random vertices uniformly
    distributed on the unit sphere.

    Keyword arguments:

    d -- the dimension
    n -- the number of random vertices

    >>> import polymake
    >>> p = polymake.rand_sphere(3, 10)
    >>> p.N_VERTICES
    10
    """
    return call_polymake_function(b"polytope", b"rand_sphere", dim, npoints)

def regular_24_cell():
    r"""Create regular 24-cell.
    """
    return call_polymake_function(b"polytope", b"regular_24_cell")

def regular_120_cell():
    r"""Create exact regular 120-cell in Q(sqrt{5}).
    """
    return call_polymake_function(b"polytope", b"regular_120_cell")

def regular_600_cell():
    r"""Create exact regular 600-cell in Q(sqrt{5}).
    """
    return call_polymake_function(b"polytope", b"regular_600_cell")



# This file was autogenerated! Do not modify!
# array interfacing Array.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class ArrayPairStringString(PmObject):
    cdef pm_Array[pair[string, string]] pm_obj

cdef class ArraySetArrayInt(PmObject):
    cdef pm_Array[pm_Set[pm_Array[int]]] pm_obj

cdef class ArrayArrayString(PmObject):
    cdef pm_Array[pm_Array[string]] pm_obj

cdef class ArrayPairStringArrayString(PmObject):
    cdef pm_Array[pair[string, pm_Array[string]]] pm_obj

cdef class ArrayString(PmObject):
    cdef pm_Array[string] pm_obj

cdef class ArrayRational(PmObject):
    cdef pm_Array[pm_Rational] pm_obj

cdef class ArrayPowerSetInt(PmObject):
    cdef pm_Array[pm_PowerSet[int]] pm_obj

cdef class ArraySetInt(PmObject):
    cdef pm_Array[pm_Set[int]] pm_obj

cdef class ArrayInt(PmObject):
    cdef pm_Array[int] pm_obj

cdef class ArrayBool(PmObject):
    cdef pm_Array[bool] pm_obj

cdef class ArrayMatrixInteger(PmObject):
    cdef pm_Array[pm_Matrix[pm_Integer]] pm_obj

cdef class ArrayArrayInt(PmObject):
    cdef pm_Array[pm_Array[int]] pm_obj

cdef class ArrayArrayPairStringString(PmObject):
    cdef pm_Array[pm_Array[pair[string, string]]] pm_obj



# This file was autogenerated! Do not modify!
# matrix interfacing Matrix.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class PmMatrix(PmObject):
    cpdef resize(self, int nr, int nc)
    cpdef int rows(self)
    cpdef int cols(self)

    cpdef _set_list_of_lists(self, data)
    cpdef _set_matrix(self, mat)
    cpdef _set_plain_list(self, mat)

cdef class MatrixFloat(PmMatrix):
    cdef pm_Matrix[float] pm_obj

cdef class MatrixInt(PmMatrix):
    cdef pm_Matrix[int] pm_obj

cdef class MatrixRational(PmMatrix):
    cdef pm_Matrix[pm_Rational] pm_obj

cdef class MatrixInteger(PmMatrix):
    cdef pm_Matrix[pm_Integer] pm_obj

cdef class MatrixQuadraticExtension(PmMatrix):
    cdef pm_Matrix[pm_QuadraticExtension] pm_obj



import unittest
import random
import operator

# NOTE: apparently this has to be done *before* importing polymake.
# Otherwise it produces segfault...
try:
    import sage.all
    import sage
except ImportError:
    sage = None

import polymake

try:
    import gmpy2
except ImportError:
    gmpy2 = None


class TestCoercion(unittest.TestCase):
    def assertCoerceBinop(self, a, b, res, op):
        ans = op(a, b)
        self.assertEqual(type(ans), type(res))
        self.assertEqual(ans, res)

    def assertCoerceBinopCommutative(self, a, b, res, op):
        self.assertCoerceBinop(a, b, res, op)
        self.assertCoerceBinop(b, a, res, op)

    def randint(self):
        if random.random() < 0.5:
            return random.choice([0, -1, 1, 2**32, -2**32, 2**64, -2**64,
                2**32-1, 2**32+1, -2**32-1, -2**32+1,
                2**64-1, 2**64+1, -2**64-1, -2**64+1])
        else:
            return random.randint(-2**100, 2**100)

    def randint_nonzero(self):
        if random.random() < 0.5:
            return random.choice([-1, 1, 2**32, -2**32, 2**64, -2**64,
                2**32-1, 2**32+1, -2**32-1, -2**32+1,
                2**64-1, 2**64+1, -2**64-1, -2**64+1])
        else:
            z = random.randint(-2**100, 2**100)
            if not z:
                return 1

    def randint_positive(self):
        if random.random() < 0.5:
            return random.choice([1, 2**32, 2**64,
                2**32-1, 2**32+1, 2**64-1, 2**64+1])
        else:
            z = random.randint(1, 2**100)
            if not z:
                return 1

    def test_int_integer(self):
        for _ in range(20):
            a = self.randint()
            b = self.randint()
            c = a + b
            self.assertCoerceBinopCommutative(polymake.Integer(a), b, polymake.Integer(a+b), operator.add)
            self.assertCoerceBinopCommutative(polymake.Integer(a), b, polymake.Integer(a*b), operator.mul)
            self.assertCoerceBinop(polymake.Integer(a), b, polymake.Integer(a-b), operator.sub)
            self.assertCoerceBinop(a, polymake.Integer(b), polymake.Integer(a-b), operator.sub)
            # TODO: not working for negative b
            if a >= 0 and b > 0:
                self.assertCoerceBinop(polymake.Integer(a), b, polymake.Integer(a//b), operator.floordiv)
                self.assertCoerceBinop(a, polymake.Integer(b), polymake.Integer(a//b), operator.floordiv)

    def test_integer_rational(self):
        for _ in range(20):
            a = polymake.Integer(self.randint())
            b = polymake.Rational(self.randint(), self.randint_positive())

            s = polymake.Rational(a*b.denominator() + b.numerator(), b.denominator())
            self.assertCoerceBinopCommutative(a, b, s, operator.add)

            p = polymake.Rational(a * b.numerator(), b.denominator())
            self.assertCoerceBinopCommutative(a, b, p, operator.mul)

            d = polymake.Rational(a*b.denominator() - b.numerator(), b.denominator())
            self.assertCoerceBinop(a, b, d, operator.sub)
            d = polymake.Rational(b.numerator() - a*b.denominator(), b.denominator())
            self.assertCoerceBinop(b, a, d, operator.sub)

    def test_integer_quadratic_extension(self):
        self.assertCoerceBinopCommutative(polymake.Integer(2),
                                          polymake.QuadraticExtension(1,2,3),
                                          polymake.QuadraticExtension(3,2,3),
                                          operator.add)

    def test_rational_quadratic_extension(self):
        self.assertCoerceBinopCommutative(polymake.Rational(2,3),
                         polymake.QuadraticExtension(1,2,3),
                         polymake.QuadraticExtension(polymake.Rational(5,3), 2, 3),
                         operator.add)

    def test_pm_conversion(self):
        from polymake.coercion import as_pm_object
        self.assertEqual(type(as_pm_object(3)), polymake.Integer)
        self.assertEqual(type(as_pm_object(2**100)), polymake.Integer)

if __name__ == '__main__':
    unittest.main()



###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from libcpp.string cimport string

from .cygmp.mpz cimport *
from .cygmp.mpz cimport *
from .defs cimport *
from .perl_object cimport wrap_perl_object, get_properties
from .integer cimport Integer
from .rational cimport Rational
from .quadratic_extension cimport QuadraticExtension
from .matrix cimport MatrixRational, MatrixQuadraticExtension

def Polytope(typ='Rational', **args):
    r"""
    Construct a polytope

    INPUT:

    - ``VERTICES`` -- a matrix of vertices

    - ``POINTS`` -- a matrix of points

    - ``FACETS`` -- a matrix of facets

    Examples:

    >>> import polymake
    >>> polymake.Polytope(POINTS=[[1,0,0],[0,1,0],[0,0,1]])
    Polytope<Rational>
    >>> polymake.Polytope(VERTICES=[[1,-1,-1],[1,1,-1],[1,-1,1],[1,1,1]], LINEALITY_SPACE=[])
    Polytope<Rational>
    >>> polymake.Polytope(INEQUALITIES=[[5,-4,0,1],[-3,0,-4,1],[-2,1,0,0],[-4,4,4,-1],[0,0,1,0],[8,0,0,-1],[1,0,-1,0],[3,-1,0,0]])
    Polytope<Rational>
    >>> polymake.Polytope(INEQUALITIES=[[1,1,0,0],[1,0,1,0],[1,-1,0,0],[1,0,-1,0]], EQUATIONS=[[0,0,0,1],[0,0,0,2]])
    Polytope<Rational>
    """
    cdef string cprop

    cdef string ctyp = <string> (b"polytope::Polytope<" + typ.encode('ascii') + b">")
    cdef pm_PerlObject * pm_obj = new pm_PerlObject(ctyp)

    for prop, value in args.items():
        cprop = prop.encode('ascii')

        if cprop == b"CONE_AMBIENT_DIM":
            pm_assign_Int(pm_obj.take(cprop), value)

        elif cprop in [b"VERTICES", b"POINTS", b"FACETS", b"INEQUALITIES", b"INPUT_LINEALITY",
                b"LINEALITY_SPACE", b"INEQUALITIES", b"EQUATIONS"]:

            if typ == "Rational":
                pm_assign_Matrix(pm_obj.take(cprop),
                                 (<MatrixRational> MatrixRational(value)).pm_obj)
            elif typ == "QuadraticExtension":
                pm_assign_Matrix(pm_obj.take(cprop),
                                 (<MatrixQuadraticExtension> MatrixQuadraticExtension(value)).pm_obj)
            else:
                raise ValueError
        else:
            raise ValueError("{} is not a valid property of polytope::Polytope<Rational>".format(cprop))

    return wrap_perl_object(pm_obj[0])

def PolytopeRational(**opts):
    return Polytope(b"Rational", **opts)

def PolytopeQuadraticExtension(**opts):
    return Polytope(b"QuadraticExtension", **opts)

def PointConfiguration(**args):
    cdef pm_Matrix[pm_Rational] * pm_mat
    cdef pm_PerlObject * pm_obj = new pm_PerlObject(b"PointConfiguration<Rational>")



"""
Generate the types handled by pypolymake.
"""
atomic_types = {
    "Bool":
    {
        "name"  : "Bool",
        "simple": True,
        "module": None,
        "perl"  : "Bool",
        "cpp"   : "bool",
        "cython": "bool",
        "cppcython": "bool"
    },

    "Int":
    {
        "name"   : "Int",
        "simple" : True,
        "module" : None,
        "perl"   : "Int",
        "cpp"    : "int",
        "cython" : "int",
        "cppcython": "int"
    },

    "Float":
    {
        "name"   : "Float",
        "simple" : True,
        "module" : None,
        "perl"   : "Float",
        "cpp"    : "float",
        "cython" : "float",
        "cppcython": "float"
    },


    "String":
    {
        "name"   : "String",
        "simple" : True,
        "module" : None,
        "perl"   : "String",
        "cpp"    : "std::string",
        "cython" : "string",
        "cppcython": "string",
    },

    "Integer":
    {
        "parent" : "PmObject",
        "name"   : "Integer",
        "simple" : False,
        "module" : "Integer",
        "perl"   : "Integer",
        "cpp"    : "Integer",
        "cython" : "Integer",
        "cppcython": "pm_Integer"
    },

    "Rational":
    {
        "parent" : "PmObject",
        "name"   : "Rational",
        "simple" : False,
        "module" : "Rational",
        "perl"   : "Rational",
        "cpp"    : "Rational",
        "cython" : "Rational",
        "cppcython": "pm_Rational"
    },

    "QuadraticExtension":
    {
        "parent" : "PmObject",
        "name"   : "QuadraticExtension",
        "simple" : False,
        "module" : "QuadraticExtension",
        "perl"   : "QuadraticExtension<Rational>",
        "cpp"    : "QuadraticExtension",
        "cython" : "QuadraticExtension",
        "cppcython": "pm_QuadraticExtension"
    },

    "PairStringString":
    {
        "name"   : "PairStringString",
        "simple" : False,
        "module" : None,
        "perl"   : "Pair<String, String>",
        "cpp"    : "std::pair<std::string, std::string>",
        "cython" : "PairStringString",
        "cppcython" : "pair[string, string]",
    },

    "PairStringArrayString":
    {
        "name"   : "PairStringArrayString",
        "simple" : False,
        "module" : None,
        "perl"   : "Pair<String, Array<String>>",
        "cpp"    : "std::pair<std::string, Array<std::string>>",
        "cython" : "PairStringArrayString",
        "cppcython": "pair[string, pm_Array[string]]",
    }
}

Scalars = ["Int", "Float", "Integer", "Rational", "QuadraticExtension"]

module_data = {
# unique types
    "Integer":  None,
    "Rational": None,
    "IncidenceMatrix": None,

# vectors and matrices
    "Vector": Scalars,
    "Matrix": Scalars,

    "SparseMatrix":
    [
        "Int",
        "Rational"
    ],

# polynomials, rational functions

    "Polynomial":
    [
        ("Rational", "Int")
    ],

    "RationalFunction":
    [
        ("Rational", "Int")
    ],

# containers
    "Array":
    [
        "Bool",
        "Int",
        "Rational",
        "String",
        "ArrayInt",
        "ArrayString",
        "SetInt",
        "PairStringString",
        "MatrixInteger",
        "ArrayPairStringString",
        "PairStringArrayString",
        "PowerSetInt",
        "SetArrayInt"
    ],

    "Set":
    [
        "Int",
        "SetInt",
        "MatrixRational",
        "ArrayInt"
    ],

    "PowerSet":
    [
        "Int"
    ],

# maps
    "Map":
    [
        ("String", "String"),
        ("Rational", "Rational"),
        ("Int", "Int"),
        ("Integer", "Int")
    ],
}


def caml_to_python(s):
    if not s or not s[0].isupper():
        raise ValueError("not Caml case")
    cap = [i for i,j in enumerate(s) if j.isupper()]
    cap.append(len(s))
    return '_'.join(s[cap[i]].lower() + s[cap[i]+1:cap[i+1]] for i in range(len(cap)-1))
def python_to_caml(s):
    und = [0]
    und.extend(i for i,j in enumerte(s) if j == '_')
    und.append(len(s))
    return ''.join(s[und[i+1]].upper() + s[und[i+1]+1:und[i+1]] for i in range(len(und)-1))

def pm_types():
    r"""
    Construct the list of supported polymake types from the dictionaries ``atomic_types`` and
    ``module_data``.

    OUTPUT:

    A dictionary:

      name -> dictionary of properties
    """
    ans = atomic_types.copy()

    for scal in module_data["Vector"]:
        cython = "Vector{scal}".format(scal=scal)
        cppcython = "pm_Vector[{scal}]".format(scal=atomic_types[scal]["cppcython"])
        perl = "Vector<{scal}>".format(scal=scal)
        cpp = "Vector<{scal}>".format(scal=atomic_types[scal]["cpp"])

        ans[cython] = {
            "parent": "PmObject",
            "name"  : cython,
            "simple": False,
            "module": "Vector",
            "cython": cython,
            "cppcython": cppcython,
            "perl"  : perl,
            "cpp"   : cpp}

    for scal in module_data["Matrix"]:
        cppcython = "pm_Matrix[{scal}]".format(scal=atomic_types[scal]["cppcython"])
        cython = "Matrix{scal}".format(scal=scal)
        perl = "Matrix<{scal}>".format(scal=atomic_types[scal]["perl"])
        cpp = "Matrix<{scal}>".format(scal=atomic_types[scal]["cpp"])

        ans[cython] = {
            "parent": "PmMatrix",
            "name"  : cython,
            "simple": False,
            "module": "Matrix",
            "cython": cython,
            "cppcython": cppcython,
            "perl"  : perl,
            "cpp"   : cpp}

    for scal in module_data["SparseMatrix"]:
        cppcython = "pm_SparseMatrix[{scal}]".format(scal=atomic_types[scal]["cppcython"])
        cython = "SparseMatrix{scal}".format(scal=scal)
        perl = "SparseMatrix<{scal}>".format(scal=atomic_types[scal]["perl"])
        cpp = "SparseMatrix<{scal}>".format(scal=atomic_types[scal]["cpp"])

        ans[cython] = {
            "parent": "PmObject",
            "name"  : cython,
            "simple": False,
            "module": "SparseMatrix",
            "cython": cython,
            "cppcython": cppcython,
            "perl"  : perl,
            "cpp"   : cpp}

    for (coeff, exp) in module_data["Polynomial"]:
        cython = "UniPolynomial{coeff}{exp}".format(coeff=coeff, exp=exp)
        perl = "UniPolynomial<{coeff}, {exp}>".format(coeff=coeff, exp=exp)
        cpp = "UniPolynomial<{coeff}, {exp}>".format(coeff=atomic_types[coeff]["cpp"],
                                                     exp=atomic_types[exp]["cpp"])

        ans[cython] = {
            "parent": "PmObject",
            "name"  : cython,
            "simple": False,
            "module": "Polynomial",
            "cython": cython,
            "cppcython": "pm_{}".format(cython),
            "perl"  : perl,
            "cpp"   : cpp}

    for (coeff, exp) in module_data["RationalFunction"]:
        cython = "RationalFunction{coeff}{exp}".format(coeff=coeff, exp=exp)
        perl = "RationalFunction<{coeff}, {exp}>".format(coeff=coeff, exp=exp)
        cpp = "RationalFunction<{coeff}, {exp}>".format(coeff=atomic_types[coeff]["cpp"],
                                                        exp=atomic_types[exp]["cpp"])

        ans[cython] = {
            "parent": "PmObject",
            "name"  : cython,
            "simple": False,
            "module": "RationalFunction",
            "cython": cython,
            "cppcython": "pm_{}".format(cython),
            "perl"  : perl,
            "cpp"   : cpp}

    ans["IncidenceMatrixNonSymmetric"] = {
        "parent": "PmObject",
        "name"  : "IncidenceMatrixNonSymmetric",
        "simple": False,
        "module": "IncidenceMatrix",
        "cython": "IncidenceMatrixNonSymmetric",
        "cppcython": "pm_IncidenceMatrixNonSymmetric",
        "perl"  : "IncidenceMatrix<NonSymmetric>",
        "cpp"   : "IncidenceMatrix<NonSymmetric>"
        }

    # containers
    todo = set((c,s) for c in ("Array","Set","PowerSet") for s in module_data[c])

    while todo:
        todo2 = set()
        while todo:
            c, s = todo.pop()
            if s not in ans:
                todo2.add((c,s))
                continue
            
            data = ans[s]
            
            cppcython = "pm_{container}[{subtype}]".format(container=c, subtype=data["cppcython"])
            cython = "{container}{subtype}".format(container=c, subtype=s)
            perl = "{container}<{subtype}>".format(container=c, subtype=data["perl"])
            cpp = "{container}<{subtype}>".format(container=c, subtype=data["cpp"])

            ans[cython] = {
                "parent": "PmObject",
                "name"  : cython,
                "simple": False,
                "module": c,
                "cython": cython,
                "cppcython": cppcython,
                "perl"  : perl,
                "cpp"   : cpp}

        todo = todo2

    # map
    for s1,s2 in module_data["Map"]:
        data1 = ans[s1]
        data2 = ans[s2]
        cppcython = "pm_Map[{s1},{s2}]".format(s1=data1["cppcython"], s2=data2["cppcython"])
        cython = "Map{s1}{s2}".format(s1=s1,s2=s2)
        perl = "Map<{s1}, {s2}>".format(s1=data1["perl"], s2=data2["perl"])
        cpp = "Map<{s1}, {s2}>".format(s1=data1["cpp"], s2=data2["cpp"])

        ans[cython] = {
            "parent": "PmObject",
            "name" : cython,
            "simple": False,
            "module": "Map",
            "cython": cython,
            "cppcython": cppcython,
            "perl" : perl,
            "cpp": cpp}

    return ans

def pm_modules():
    ans = set(typ["module"] for typ in pm_types().values())
    ans.remove(None)
    return sorted(caml_to_python(x) for x in ans)



###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from cython.operator cimport dereference, preincrement

from libcpp cimport bool
from libcpp.string cimport string

from .integer cimport Integer
from .rational cimport Rational

cdef class MapStringString(PmObject):
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, bytes key):
        return self.pm_obj[key]

    def __len__(self):
        return self.pm_obj.size()

    def __iter__(self):
        cdef pm_Map[string,string].iterator it = self.pm_obj.begin()
        while not it.at_end():
            yield dereference(it).first
            preincrement(it)

    def __contains__(self, bytes key):
        return self.pm_obj.exists(key)

    def items(self):
        "Iterator throug the pairs (key, value) of this map"
        cdef pm_Map[string,string].iterator it = self.pm_obj.begin()
        while not it.at_end():
            yield (dereference(it).first, dereference(it).second)
            preincrement(it)

    def keys(self):
        "Iterator throug the keys of this map"
        return iter(self)

    def values(self):
        "Iterator throug the values of this map"
        cdef pm_Map[string,string].iterator it = self.pm_obj.begin()
        while not it.at_end():
            yield dereference(it).second
            preincrement(it)

    def python(self):
        return dict(self.items())

cdef class MapIntInt(PmObject):
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, int key):
        return self.pm_obj[key]

    def __len__(self):
        return self.pm_obj.size()

    def __iter__(self):
        cdef pm_Map[int,int].iterator it = self.pm_obj.begin()
        while not it.at_end():
            yield dereference(it).first
            preincrement(it)

    def __contains__(self, int key):
        r"""
        TESTS:

        >>> import polymake
        >>> c = polymake.associahedron(3)
        >>> m = c.TWO_FACE_SIZES
        >>> 4 in m
        True
        >>> 5 in m
        True
        >>> 2 in m
        False
        >>> 'a' in m
        Traceback (most recent call last):
        ...
        TypeError: an integer is required
        """
        return self.pm_obj.exists(key)

    def items(self):
        "Iterator throug the pairs (key, value) of this map"
        cdef pm_Map[int,int].iterator it = self.pm_obj.begin()
        while not it.at_end():
            yield (dereference(it).first, dereference(it).second)
            preincrement(it)

    def keys(self):
        "Iterator throug the keys of this map"
        return iter(self)

    def values(self):
        "Iterator throug the values of this map"
        cdef pm_Map[int,int].iterator it = self.pm_obj.begin()
        while not it.at_end():
            yield dereference(it).second
            preincrement(it)

    def python(self):
        return dict(self.items())


cdef class MapRationalRational(PmObject):
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, Rational key):
        cdef Rational out = Rational.__new__(Rational)
        out.pm_obj = self.pm_obj[key.pm_obj]
        return out

    def __len__(self):
        return self.pm_obj.size()

    def __contains__(self, Rational key):
        r"""
        TESTS:

        >>> import polymake
        >>> c = polymake.associahedron(3)
        >>> m = c.RELATIVE_VOLUME
        >>> polymake.Rational(805,1) in m
        True
        >>> polymake.Rational(805,2) in m
        False
        """
        return self.pm_obj.exists(key.pm_obj)


cdef class MapIntegerInt(PmObject):
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, Integer key):
        return self.pm_obj[key.pm_obj]

    def __len__(self):
        return self.pm_obj.size()

    def __contains__(self, Integer key):
        return self.pm_obj.exists(key.pm_obj)




#!/usr/bin/env python3

import unittest
import random

# NOTE: apparently this has to be done *before* importing polymake.
# Otherwise it produces segfault...
try:
    import sage.all
    import sage
except ImportError:
    sage = None

import polymake

try:
    import gmpy2
except ImportError:
    gmpy2 = None

class TestPolymakeRational(unittest.TestCase):
    def test_init(self):
        self.assertTrue(polymake.Rational(-2,4) ==
                        polymake.Rational(2,-4) ==
                        polymake.Rational(-1,2) ==
                        polymake.Rational(1,-2))

        with self.assertRaises(ValueError):
             polymake.Rational(0,0)
        with self.assertRaises(ValueError):
            polymake.Rational(2,0)

        for num in [-1, 0, 1, 2**32, 2**32-1, 2**32+1,
                2**64, 2**64-1, 2**64+1]:
            polymake.Rational(num)
            for den in [1, 2**32, 2**32-1, 2**32+1, 2**64, 2**64-1, 2**64+1]:
                polymake.Rational(num, den)

    def test_init_with_sage_input(self):
        if sage is None:
            return

        from sage.all import Integer, Rational
        self.assertEqual(polymake.Rational(Integer(5)), polymake.Rational(5))
        self.assertEqual(polymake.Rational(Rational((2,3))), polymake.Rational(2,3))

    def test_init_with_gmpy2_input(self):
        if gmpy2 is None:
            return

        self.assertEqual(polymake.Rational(gmpy2.mpz(3), gmpy2.mpz(5)), polymake.Rational(3, 5))
        self.assertEqual(polymake.Rational(gmpy2.mpq(3, 4)), polymake.Rational(3, 4))

    def test_bool(self):
        self.assertEqual(bool(polymake.Rational(0,1)), False)
        self.assertEqual(bool(polymake.Rational(1,1)), True)
        self.assertEqual(bool(polymake.Rational(-1,2)), True)
        self.assertEqual(bool(polymake.Rational(2,3)), True)
        self.assertEqual(bool(polymake.Rational(-2,1)), True)

    def test_idempotent(self):
        for a in [(1,3),(0,1),(2**100+1, 3**100-2)]:
            b = polymake.Rational(a)
            self.assertEqual(polymake.Rational(b), b)

    def test_sage(self):
        if sage is None:
            return

        # sage bug...
        #for a in [(-1,3), (2**100,3**60)]:
        for a in [(-1,3)]:
            b = polymake.Rational(a).sage()
            c = sage.all.Rational(a)
            self.assertEqual(type(b), type(c))
            self.assertEqual(b, c)

    def test_gmpy2(self):
        if gmpy2 is None:
            return

        for num, den in [(-1,3), (2**100,3**60)]:
            b = gmpy2.mpq(polymake.Rational(num, den))
            c = gmpy2.mpq(num, den)
            self.assertEqual(type(b), type(c))
            self.assertEqual(b, c)

    def test_big(self):
        num = 2**97 - 1
        den = 3**100 + 1
        a = polymake.Rational(num, den)
        self.assertEqual(a.numerator(), num)
        self.assertEqual(a.denominator(), den)

    def test_num_den(self):
        self.assertEqual(polymake.Rational(2, 5).numerator(), 2)
        self.assertEqual(polymake.Rational(2, 5).denominator(), 5)

    def test_cmp(self):
        a = 2
        b = 3
        pa = polymake.integer.Integer(2)
        pb = polymake.integer.Integer(3)
        qa = polymake.Rational(2)
        qb = polymake.Rational(3)
        self.assertTrue(qa == qa and pa == qa and qa == pa and qa == a and a == qa)

        self.assertTrue(qa != qb and pa != qb and qa != pb and qa != b and a != qb)
        self.assertTrue(qa <  qb and pa <  qb and qa <  pb and qa <  b and a <  qb)
        self.assertTrue(qa <= qb and pa <= qb and qa <= pb and qa <= b and a <  qb)

        self.assertTrue(qb >= qa and pb >= qa and qb >= pa and qb >= a and b >= qa)
        self.assertTrue(qb >  qa and pb >  qa and qb >  pa and qb >  a and b >= qa)

        self.assertFalse(qa != qa or pa != qa or qa != pa or qa != a or a != qa)

        self.assertFalse(qa == qb or pa == qb or qa == pb or qa == b or a == qb)
        self.assertFalse(qa >= qb or pa >= pb or pa >= pb or qa >= b or a >= qb)
        self.assertFalse(qa >  qb or pa >  pb or pa >  pb or qa >  b or a >  qb)

        self.assertFalse(qb <= qa or pb <= qa or qb <= pa or qb <= a or b <= qa)
        self.assertFalse(qb <  qa or pb <  qa or qb <  pa or qb <  a or b <  qa)

    def test_binop(self):
        a = polymake.Rational(1,2)
        b = polymake.Rational(2,3)
        self.assertTrue(a + b == polymake.Rational(7,6))

        for _ in range(100):
            anum = random.randint(-10,10)
            aden = random.randint(1,10)
            bnum = random.randint(-10,10)
            bden = random.randint(1,10)
            a = polymake.Rational(anum, aden)
            b = polymake.Rational(bnum, bden)
            self.assertTrue(a + b == polymake.Rational(anum*bden + bnum*aden, aden*bden))
            self.assertTrue(a - b == polymake.Rational(anum*bden - bnum*aden, aden*bden))
            self.assertTrue(a * b == polymake.Rational(anum*bnum, aden*bden))
            if bnum:
                self.assertTrue(a / b == polymake.Rational(anum*bden, aden*bnum))

            ones = [1, polymake.integer.Integer(1)]
            if sage is not None:
                ones.append(sage.all.Integer(1))
                ones.append(sage.all.Rational(1))
            # Note: gmpy2.mpq seems a bit broken... see also
            # https://trac.sagemath.org/ticket/28394
            # if gmpy2 is not None:
            #     ones.append(gmpy2.mpz(1))
            #     ones.append(gmpy2.mpq(1))
            for o in ones:
                msg = "a = {}  type(o) = {}".format(a, type(o))
                self.assertEqual(a + o, polymake.Rational(anum+aden, aden), msg)
                self.assertEqual(o + a, polymake.Rational(anum+aden, aden), msg)
                self.assertEqual(a - o, polymake.Rational(anum-aden, aden), msg)
                self.assertEqual(o - a, polymake.Rational(aden-anum, aden), msg)
            twos = [2, polymake.integer.Integer(2)]
            for t in twos:
                msg = "a = {}  type(t) = {}".format(a, type(t))
                self.assertEqual(a * t, polymake.Rational(2*anum, aden), msg)
                self.assertEqual(t * a, polymake.Rational(anum*2, aden), msg)
                self.assertEqual(a / t, polymake.Rational(anum, aden*2), msg)
                if anum:
                    self.assertTrue(t / a == polymake.Rational(2*aden, anum), msg)

    def test_zero_division(self):
        zeros = [0, polymake.integer.Integer(0), polymake.Rational(0)]
        rats = [polymake.Rational(0,1), polymake.Rational(1,1)]
        for z in zeros:
            for r in rats:
                with self.assertRaises(ZeroDivisionError):
                    r / z

if __name__ == '__main__':
    unittest.main()



import unittest

# NOTE: apparently this has to be done *before* importing polymake.
# Otherwise it produces segfault...
try:
    import sage.all
    import sage
except:
    sage = None

import polymake

class TestPolytopeQuadraticExtension(unittest.TestCase):
    def test_from_sage(self):
        if sage is None:
            return

        from sage.all import polytopes
        for p_sage in [polytopes.dodecahedron(), polytopes.icosahedron()]:
            p_pm = polymake.polymake(p_sage)
            self.assertEqual(p_pm.F_VECTOR.python(), list(p_sage.f_vector())[1:-1])

if __name__ == '__main__':
    unittest.main()



###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from libcpp.string cimport string
from libcpp.pair cimport pair

from .defs cimport pm_Array
from .array cimport ArrayString, ArrayArrayPairStringString

# properly initialize the Main
# FIXME: pass user-settings parameter?
cdef Main * pm = new Main("@interactive")
pm.set_application(b"polytope")
pm.pm_include(b"common::sage.rules")

cdef list apps = [b"common", b"fulton", b"group", b"matroid", b"topaz",
    b"fan", b"graph", b"ideal", b"polytope", b"tropical"]

def pm_set_application(bytes s):
    pm.set_application(s)

def pm_include(bytes s):
    pm.pm_include(s)

cdef extern from "polymake/client.h" namespace "polymake":
    pm_Array[string] call_it "call_function" (string) except +

def functions_in_current_application():
    cdef ArrayString s = ArrayString.__new__(ArrayString)
    s.pm_obj = call_it(<string> "Sage::functions_for_object")
    return s.python()

cdef extern from "polymake/client.h" namespace "polymake":
    pm_Array[pm_Array[pair[string,string]]] call_it2 "call_function" (string, string) except +

def arguments(bytes fname):
    r"""
    Return a list of list of pairs of strings.
    """
    cdef ArrayArrayPairStringString a = ArrayArrayPairStringString.__new__(ArrayArrayPairStringString)
    a.pm_obj = call_it2(<string> "Sage::arguments", <string> fname)
    return a.python()




# This file was autogenerated! Do not modify!
# rational_function interfacing RationalFunction.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class RationalFunctionRationalInt(PmObject):
    cdef pm_RationalFunctionRationalInt pm_obj



###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016      Vincent Delecroix <vincent.delecroix@labri.fr>
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from .defs cimport *
from .pm_object cimport PmObject

cdef class PerlObject(PmObject):
    cdef pm_PerlObject * pm_obj       # underlying perl object
    cdef ref                          # reference to other perl object
    cdef dict properties              # polymake properties
    cdef dict methods                 # polymake methods

cdef PerlObject wrap_perl_object(pm_PerlObject pm_obj)
cpdef dict get_properties(PerlObject p)




###############################################################################
#       Copyright (C) 2017-2019  Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from libcpp.pair cimport pair

from cython.operator cimport dereference, preincrement, postincrement

from .defs cimport ostringstream, pm_repr_wrap
from .set cimport SetInt

cdef class ArrayBool(PmObject):
    def __len__(self): return self.pm_obj.size()
    def __getitem__(self, Py_ssize_t i):
        if i < 0 or i >= self.pm_obj.size(): raise IndexError
        return self.pm_obj[i]
    def __iter__(self):
        cdef pm_Array[bool].const_iterator it = self.pm_obj.begin()
        while it != self.pm_obj.end():
            yield dereference(it)
            postincrement(it)
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')
    def python(self):
        r"""
        Converts into a list of booleans
        """
        return [self.pm_obj[i] for i in range(len(self))]

cdef class ArrayInt(PmObject):
    def __len__(self): return self.pm_obj.size()
    def __getitem__(self, Py_ssize_t i):
        r"""
        >>> import polymake
        >>> p = polymake.cube(3)
        >>> type(c)
        <type 'polymake.array.ArrayInt'>
        >>> c[0]
        7
        >>> c[-1]
        Traceback (most recent call last):
        ...
        IndexError
        >>> c[len(c)]
        Traceback (most recent call last):
        ...
        IndexError
        """
        if i < 0 or i >= self.pm_obj.size(): raise IndexError
        return self.pm_obj[i]
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')
    def python(self):
        r"""
        Converts into a Python list of integers

        >>> import polymake
        >>> p = polymake.cube(3)
        >>> type(c)
        <type 'polymake.array.ArrayInt'>
        >>> c.python()
        [7, 6, 5, 4, 3, 2, 1, 0]
        """
        return [self[i] for i in range(len(self))]

cdef class ArrayString(PmObject):
    def __len__(self): return self.pm_obj.size()
    def __getitem__(self, Py_ssize_t i):
        if i < 0 or i >= self.pm_obj.size(): raise IndexError
        return (<bytes>self.pm_obj[i]).decode('ascii')
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')
    def python(self):
        return [self[i] for i in range(len(self))]

cdef class ArraySetInt(PmObject):
    r"""
    Array of sets of integers

    EXAMPLES:

    >>> import polymake
    >>> c = polymake.cube(3,2,0)
    """
    def __len__(self): return self.pm_obj.size()
    def __getitem__(self, Py_ssize_t i):
        if i < 0 or i >= self.pm_obj.size(): raise IndexError
        cdef SetInt s = SetInt.__new__(SetInt)
        s.pm_obj = self.pm_obj[i]
        return s
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

cdef class ArrayArrayInt(PmObject):
    def __len__(self): return self.pm_obj.size()
    def __getitem__(self, Py_ssize_t i):
        if i < 0 or i >= self.pm_obj.size(): raise IndexError
        cdef ArrayInt a = ArrayInt.__new__(ArrayInt)
        a.pm_obj = self.pm_obj[i]
        return a
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

cdef class ArrayArrayString(PmObject):
    def __len__(self): return self.pm_obj.size()
    def __getitem__(self, Py_ssize_t i):
        if i < 0 or i >= self.pm_obj.size(): raise IndexError
        cdef ArrayString a = ArrayString.__new__(ArrayString)
        a.pm_obj = self.pm_obj[i]
        return a
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')
    def python(self):
        r"""Converts into a list of list of strings
        """
        return [self[i].python() for i in range(len(self))]

cdef class ArrayPairStringString(PmObject):
    def __len__(self): return self.pm_obj.size()
    def __getitem__(self, Py_ssize_t i):
        if i < 0 or i >= self.pm_obj.size(): raise IndexError
        cdef pair[string,string] x = self.pm_obj[i]
        return (x.first, x.second)
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')
    def python(self):
        cdef Py_ssize_t i
        return [self[i] for i in range(self.pm_obj.size())]

cdef class ArrayPairStringArrayString(PmObject):
    def __len__(self): return self.pm_obj.size()
    def __getitem__(self, Py_ssize_t i):
        if i < 0 or i >= self.pm_obj.size(): raise IndexError
        cdef pair[string, pm_Array[string]] x = self.pm_obj[i]
        cdef ArrayString y = ArrayString.__new__(ArrayString)
        y.pm_obj = x.second
        return (x.first, y)
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')
    def python(self):
        cdef Py_ssize_t i
        l = [self[i] for i in range(self.pm_obj.size())]
        return [(x, y.python()) for x,y in l]

cdef class ArrayArrayPairStringString(PmObject):
    def __len__(self): return self.pm_obj.size()
    def __getitem__(self, Py_ssize_t i):
        if i < 0 or i >= self.pm_obj.size(): raise IndexError
        cdef ArrayPairStringString a = ArrayPairStringString.__new__(ArrayPairStringString)
        a.pm_obj = self.pm_obj[i]
        return a
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')
    def python(self):
        cdef Py_ssize_t i
        return [self[i].python() for i in range(self.pm_obj.size())]



###############################################################################
#       Copyright (C) 2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr> 
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from cpython.object cimport Py_LT, Py_LE, Py_EQ, Py_NE, Py_GT, Py_GE
from cpython.long cimport PyLong_AsLong

from .cygmp.types cimport mpz_t, mpq_t, mpz_srcptr, mpq_srcptr
from .cygmp.mpz cimport mpz_init, mpz_clear, mpz_set_si, mpz_set
from .cygmp.mpq cimport mpq_init, mpq_clear, mpq_numref, mpq_denref, mpq_canonicalize
from .cygmp.utils cimport mpz_set_pylong, mpz_get_bytes, mpq_get_bytes

from libcpp.string cimport string

from .defs cimport pm_Integer, ostringstream, pm_repr
from .integer cimport Integer

def get_num_den(elt):
    if isinstance(elt, (tuple, list)):
        if len(elt) == 2:
            num, den = elt
        else:
            raise ValueError("no conversion to rational")
        if not isinstance(num, numbers.Integral) or not isinstance(den, numbers.Integral):
            raise ValueError("no conversion to rational")

    elif isinstance(elt, numbers.Rational):
        num = elt.numerator
        den = elt.denominator

        # Sage rational are not complient with Numbers abc
        if callable(num) and callable(den):
            num = num()
            den = den()
    else:
        raise TypeError("no conversion to rational")

    return (num, den)

cdef class Rational(PmObject):
    r"""Polymake rational

    >>> import polymake
    >>> polymake.Rational(3)
    3
    >>> polymake.Rational(2, 7)
    2/7
    >>> polymake.Rational((2,7))
    2/7

    >>> polymake.Rational(polymake.Integer(-3))
    -3
    >>> polymake.Rational(polymake.Integer(-3), polymake.Integer(19))
    -3/19
    >>> polymake.Rational(polymake.Integer(-3), 19)
    -3/19
    >>> polymake.Rational(-3, polymake.Integer(19))
    -3/19

    >>> polymake.Rational(polymake.Rational(2, 5))
    2/5

    >>> polymake.Rational(polymake.Rational(2, 5), polymake.Rational(3,7))
    Traceback (most recent call last):
    ...
    ValueError: invalid argument for polymake Rational
    >>> polymake.Rational((polymake.Rational(1,2), polymake.Rational(2,3)))
    Traceback (most recent call last):
    ...
    ValueError: no conversion to rational
    """
    def __init__(self, num, den=None):
        if den is None:
            num, den = get_num_den(num)
        elif not isinstance(num, numbers.Integral) or not isinstance(den, numbers.Integral):
            raise ValueError("invalid argument for polymake Rational")

        if not den:
            raise ValueError("denominator must not be zero")

        cdef mpq_t z
        mpq_init(z)
        if isinstance(num, Integer):
            mpz_set(mpq_numref(z), (<Integer> num).pm_obj.get_rep())
        elif isinstance(num, long):
            mpz_set_pylong(mpq_numref(z), num)
        elif isinstance(num, int):  # Python 2 only
            mpz_set_si(mpq_numref(z), num)
        else:
            num = num.__index__()
            if isinstance(num, long):
                mpz_set_pylong(mpq_numref(z), num)
            elif isinstance(num, int):  # Python 2 only
                mpz_set_si(mpq_numref(z), num)

        if isinstance(den, Integer):
            mpz_set(mpq_denref(z), (<Integer> den).pm_obj.get_rep())
        elif isinstance(den, long):
            mpz_set_pylong(mpq_denref(z), den)
        elif isinstance(den, int):  # Python 2 only
            mpz_set_si(mpq_denref(z), den)
        else:
            den = den.__index__()
            if isinstance(den, long):
                mpz_set_pylong(mpq_denref(z), den)
            elif isinstance(den, int):  # Python 2 only
                mpz_set_si(mpq_denref(z), den)

        mpq_canonicalize(z)
        self.pm_obj.set_mpq_srcptr(<mpq_srcptr>z)
        mpq_clear(z)

    def __mpq__(self):
        # TODO: go directly through mpq initialization
        import gmpy2
        return gmpy2.mpq(int(self.numerator()), int(self.denominator()))

    def sage(self):
        r"""
        Converts this rational to Sage
        """
        from .sage_conversion import Rational_to_sage
        return Rational_to_sage(self)

    _rational_ = sage

    def python(self):
        r"""Converts to a python fraction

        >>> import polymake
        >>> c = polymake.Rational(12, 5)
        """
        from fractions import Fraction
        return Fraction(self.numerator().python(), self.denominator().python())

    def __nonzero__(self):
        return not pm_is_zero(self.pm_obj)

    cpdef _richcmp_(self, other, int op):
        cdef int c = (<Rational>self).pm_obj.compare((<Rational>other).pm_obj)
        if c < 0:
            return op in (Py_LE, Py_LT, Py_NE)
        elif c == 0:
            return op in (Py_LE, Py_EQ, Py_GE)
        else:
            return op in (Py_GE, Py_GT, Py_NE)

    def __repr__(self):
        cdef ostringstream out
        pm_repr(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def numerator(self):
        cdef Integer ans = Integer.__new__(Integer)
        cdef mpq_srcptr z = self.pm_obj.get_rep()
        ans.pm_obj.set_mpz_srcptr(mpq_numref(z))
        return ans

    def denominator(self):
        cdef Integer ans = Integer.__new__(Integer)
        cdef mpq_srcptr z = self.pm_obj.get_rep()
        ans.pm_obj.set_mpz_srcptr(mpq_denref(z))
        return ans

    cpdef _add_(self, other):
        r"""
        >>> from polymake import Integer, Rational
        >>> Rational(1,2) + Rational(1,2)
        1
        >>> Rational(1) + Integer(2)
        3
        >>> Integer(2) + Rational(1)
        3
        >>> Rational(2,3) + 318
        956/3
        >>> -2 + Rational(3,4)
        -5/4
        """
        cdef Rational ans = Rational.__new__(Rational)
        ans.pm_obj = (<Rational>self).pm_obj + (<Rational>other).pm_obj
        return ans

    cpdef _sub_(self, other):
        cdef Rational ans = Rational.__new__(Rational)
        ans.pm_obj = (<Rational>self).pm_obj - (<Rational>other).pm_obj
        return ans

    cpdef _mul_(self, other):
        cdef Rational ans = Rational.__new__(Rational)
        ans.pm_obj = (<Rational>self).pm_obj * (<Rational>other).pm_obj
        return ans

    cpdef _div_(self, other):
        r"""
        >>> from polymake import Rational
        >>> Rational(2,3) / Rational(5,7)
        14/15
        """
        cdef Rational ans = Rational.__new__(Rational)
        ans.pm_obj = (<Rational>self).pm_obj / (<Rational>other).pm_obj
        return ans

    def __pow__(self, n, mod):
        r"""
        >>> from polymake import Integer, Rational
        >>> Rational(2,3) ** 5
        32/243
        >>> Rational(2,3) ** Integer(5)
        32/243
        >>> Rational(2,3) ** (-5)
        243/32
        >>> Rational(2,3) ** (2**100)
        Traceback (most recent call last):
        ...
        OverflowError: Python int too large to convert to C long
        """
        if type(self) is not Rational:
            return NotImplemented
        cdef Rational ans = Rational.__new__(Rational)
        if isinstance(n, (int, long)):
            ans.pm_obj = pm_pow((<Rational> self).pm_obj, PyLong_AsLong(n))
        elif isinstance(n, Integer):
            ans.pm_obj = pm_pow((<Rational> self).pm_obj, <long> (<Integer> n).pm_obj)
        else:
            n = Integer(n)
            ans.pm_obj = pm_pow((<Rational> self).pm_obj, <long> (<Integer> n).pm_obj)
        return ans

import numbers
numbers.Rational.register(Rational)



# This file was autogenerated! Do not modify!
# incidence_matrix interfacing IncidenceMatrix.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class IncidenceMatrixNonSymmetric(PmObject):
    cdef pm_IncidenceMatrixNonSymmetric pm_obj



from .types cimport *
from libc.stdio cimport FILE

from libc.stdint cimport intmax_t, uintmax_t

cdef extern from "gmp.h":

    ### Integer Functions ###

    # Initialization Functions
    void mpz_init (mpz_t integer)
    void mpz_init2 (mpz_t integer, unsigned long n)
    void mpz_clear (mpz_t integer)
    void mpz_realloc2 (mpz_t integer, unsigned long n)

    # Assignment Functions
    void mpz_set (mpz_t rop, mpz_t op)
    void mpz_set_ui (mpz_t rop, unsigned long int op)
    void mpz_set_si (mpz_t rop, signed long int op)
    void mpz_set_ux (mpz_t rop, uintmax_t op)
    void mpz_set_sx (mpz_t rop, intmax_t op)
    void mpz_set_d (mpz_t rop, double op)
    void mpz_set_q (mpz_t rop, mpq_t op)
    void mpz_set_f (mpz_t rop, mpf_t op)
    int mpz_set_str (mpz_t rop, char *str, int base)
    void mpz_swap (mpz_t rop1, mpz_t rop2)

    # Combined Initialization and Assignment Functions
    void mpz_init_set (mpz_t rop, mpz_t op)
    void mpz_init_set_ui (mpz_t rop, unsigned long int op)
    void mpz_init_set_si (mpz_t rop, signed long int op)
    void mpz_init_set_ux (mpz_t rop, uintmax_t op)
    void mpz_init_set_sx (mpz_t rop, intmax_t op)
    void mpz_init_set_d (mpz_t rop, double op)
    int mpz_init_set_str (mpz_t rop, char *str, int base)

    # Conversion Functions
    unsigned long int mpz_get_ui (mpz_t op)
    signed long int mpz_get_si (mpz_t op)
    uintmax_t mpz_get_ux (mpz_t op)
    intmax_t mpz_get_sx (mpz_t op)
    double mpz_get_d (mpz_t op)
    double mpz_get_d_2exp (long int *exp, mpz_t op)
    char * mpz_get_str (char *str, int base, mpz_t op)

    # Arithmetic Functions
    void mpz_add (mpz_t rop, mpz_t op1, mpz_t op2)
    void mpz_add_ui (mpz_t rop, mpz_t op1, unsigned long int op2)
    void mpz_sub (mpz_t rop, mpz_t op1, mpz_t op2)
    void mpz_sub_ui (mpz_t rop, mpz_t op1, unsigned long int op2)
    void mpz_ui_sub (mpz_t rop, unsigned long int op1, mpz_t op2)
    void mpz_mul (mpz_t rop, mpz_t op1, mpz_t op2)
    void mpz_mul_si (mpz_t rop, mpz_t op1, long int op2)
    void mpz_mul_ui (mpz_t rop, mpz_t op1, unsigned long int op2)
    void mpz_addmul (mpz_t rop, mpz_t op1, mpz_t op2)
    void mpz_addmul_ui (mpz_t rop, mpz_t op1, unsigned long int op2)
    void mpz_submul (mpz_t rop, mpz_t op1, mpz_t op2)
    void mpz_submul_ui (mpz_t rop, mpz_t op1, unsigned long int op2)
    void mpz_mul_2exp (mpz_t rop, mpz_t op1, unsigned long int op2)
    void mpz_neg (mpz_t rop, mpz_t op)
    void mpz_abs (mpz_t rop, mpz_t op)

    # Division Functions
    void mpz_cdiv_q (mpz_t q, mpz_t n, mpz_t d)
    void mpz_cdiv_r (mpz_t r, mpz_t n, mpz_t d)
    void mpz_cdiv_qr (mpz_t q, mpz_t r, mpz_t n, mpz_t d)
    unsigned long int mpz_cdiv_q_ui (mpz_t q, mpz_t n, unsigned long int d)
    unsigned long int mpz_cdiv_r_ui (mpz_t r, mpz_t n, unsigned long int d)
    unsigned long int mpz_cdiv_qr_ui (mpz_t q, mpz_t r, mpz_t n, unsigned long int d)
    unsigned long int mpz_cdiv_ui (mpz_t n, unsigned long int d)
    void mpz_cdiv_q_2exp (mpz_t q, mpz_t n, unsigned long int b)
    void mpz_cdiv_r_2exp (mpz_t r, mpz_t n, unsigned long int b)
    void mpz_fdiv_q (mpz_t q, mpz_t n, mpz_t d)
    void mpz_fdiv_r (mpz_t r, mpz_t n, mpz_t d)
    void mpz_fdiv_qr (mpz_t q, mpz_t r, mpz_t n, mpz_t d)
    unsigned long int mpz_fdiv_q_ui (mpz_t q, mpz_t n, unsigned long int d)
    unsigned long int mpz_fdiv_r_ui (mpz_t r, mpz_t n, unsigned long int d)
    unsigned long int mpz_fdiv_qr_ui (mpz_t q, mpz_t r, mpz_t n, unsigned long int d)
    unsigned long int mpz_fdiv_ui (mpz_t n, unsigned long int d)
    void mpz_fdiv_q_2exp (mpz_t q, mpz_t n, unsigned long int b)
    void mpz_fdiv_r_2exp (mpz_t r, mpz_t n, unsigned long int b)
    void mpz_tdiv_q (mpz_t q, mpz_t n, mpz_t d)
    void mpz_tdiv_r (mpz_t r, mpz_t n, mpz_t d)
    void mpz_tdiv_qr (mpz_t q, mpz_t r, mpz_t n, mpz_t d)
    unsigned long int mpz_tdiv_q_ui (mpz_t q, mpz_t n, unsigned long int d)
    unsigned long int mpz_tdiv_r_ui (mpz_t r, mpz_t n, unsigned long int d)
    unsigned long int mpz_tdiv_qr_ui (mpz_t q, mpz_t r, mpz_t n, unsigned long int d)
    unsigned long int mpz_tdiv_ui (mpz_t n, unsigned long int d)
    void mpz_tdiv_q_2exp (mpz_t q, mpz_t n, unsigned long int b)
    void mpz_tdiv_r_2exp (mpz_t r, mpz_t n, unsigned long int b)
    void mpz_mod (mpz_t r, mpz_t n, mpz_t d)
    unsigned long int mpz_mod_ui (mpz_t r, mpz_t n, unsigned long int d)
    void mpz_divexact (mpz_t q, mpz_t n, mpz_t d)
    void mpz_divexact_ui (mpz_t q, mpz_t n, unsigned long d)
    bint mpz_divisible_p (mpz_t n, mpz_t d)
    bint mpz_divisible_ui_p (mpz_t n, unsigned long int d)
    bint mpz_divisible_2exp_p (mpz_t n, unsigned long int b)
    bint mpz_congruent_p (mpz_t n, mpz_t c, mpz_t d)
    bint mpz_congruent_ui_p (mpz_t n, unsigned long int c, unsigned long int d)
    bint mpz_congruent_2exp_p (mpz_t n, mpz_t c, unsigned long int b)

    # Exponentiation Functions
    void mpz_powm (mpz_t rop, mpz_t base, mpz_t exp, mpz_t mod)
    void mpz_powm_ui (mpz_t rop, mpz_t base, unsigned long int exp, mpz_t mod)
    void mpz_pow_ui (mpz_t rop, mpz_t base, unsigned long int exp)
    void mpz_ui_pow_ui (mpz_t rop, unsigned long int base, unsigned long int exp)

    # Root Extraction Functions
    int mpz_root (mpz_t rop, mpz_t op, unsigned long int n)
    void mpz_rootrem (mpz_t root, mpz_t rem, mpz_t u, unsigned long int n)
    void mpz_sqrt (mpz_t rop, mpz_t op)
    void mpz_sqrtrem (mpz_t rop1, mpz_t rop2, mpz_t op)
    bint mpz_perfect_power_p (mpz_t op)
    bint mpz_perfect_square_p (mpz_t op)

    # Number Theoretic Functions
    bint mpz_probab_prime_p (mpz_t n, int reps)
    void mpz_nextprime (mpz_t rop, mpz_t op)
    void mpz_gcd (mpz_t rop, mpz_t op1, mpz_t op2)
    unsigned long int mpz_gcd_ui (mpz_t rop, mpz_t op1, unsigned long int op2)
    void mpz_gcdext (mpz_t g, mpz_t s, mpz_t t, mpz_t a, mpz_t b)
    void mpz_lcm (mpz_t rop, mpz_t op1, mpz_t op2)
    void mpz_lcm_ui (mpz_t rop, mpz_t op1, unsigned long op2)
    int mpz_invert (mpz_t rop, mpz_t op1, mpz_t op2)
    int mpz_jacobi (mpz_t a, mpz_t b)
    int mpz_legendre (mpz_t a, mpz_t p)
    int mpz_kronecker (mpz_t a, mpz_t b)
    int mpz_kronecker_si (mpz_t a, long b)
    int mpz_kronecker_ui (mpz_t a, unsigned long b)
    int mpz_si_kronecker (long a, mpz_t b)
    int mpz_ui_kronecker (unsigned long a, mpz_t b)
    unsigned long int mpz_remove (mpz_t rop, mpz_t op, mpz_t f)
    void mpz_fac_ui (mpz_t rop, unsigned long int op)
    void mpz_bin_ui (mpz_t rop, mpz_t n, unsigned long int k)
    void mpz_bin_uiui (mpz_t rop, unsigned long int n, unsigned long int k)
    void mpz_fib_ui (mpz_t fn, unsigned long int n)
    void mpz_fib2_ui (mpz_t fn, mpz_t fnsub1, unsigned long int n)
    void mpz_lucnum_ui (mpz_t ln, unsigned long int n)
    void mpz_lucnum2_ui (mpz_t ln, mpz_t lnsub1, unsigned long int n)

    # Comparison Functions
    int mpz_cmp (mpz_t op1, mpz_t op2)
    int mpz_cmp_d (mpz_t op1, double op2)
    int mpz_cmp_si (mpz_t op1, signed long int op2)
    int mpz_cmp_ui (mpz_t op1, unsigned long int op2)
    int mpz_cmpabs (mpz_t op1, mpz_t op2)
    int mpz_cmpabs_d (mpz_t op1, double op2)
    int mpz_cmpabs_ui (mpz_t op1, unsigned long int op2)
    int mpz_sgn (mpz_t op)

    # Logical and Bit Manipulation Functions
    void mpz_and (mpz_t rop, mpz_t op1, mpz_t op2)
    void mpz_ior (mpz_t rop, mpz_t op1, mpz_t op2)
    void mpz_xor (mpz_t rop, mpz_t op1, mpz_t op2)
    void mpz_com (mpz_t rop, mpz_t op)
    unsigned long int mpz_popcount (mpz_t op)
    unsigned long int mpz_hamdist (mpz_t op1, mpz_t op2)
    unsigned long int mpz_scan0 (mpz_t op, unsigned long int starting_bit)
    unsigned long int mpz_scan1 (mpz_t op, unsigned long int starting_bit)
    void mpz_setbit (mpz_t rop, unsigned long int bit_index)
    void mpz_clrbit (mpz_t rop, unsigned long int bit_index)
    void mpz_combit (mpz_t rop, unsigned long int bit_index)
    int mpz_tstbit (mpz_t op, unsigned long int bit_index)

    # Input and Output Functions
    size_t mpz_out_str (FILE *stream, int base, mpz_t op)
    size_t mpz_inp_str (mpz_t rop, FILE *stream, int base)
    size_t mpz_out_raw (FILE *stream, mpz_t op)
    size_t mpz_inp_raw (mpz_t rop, FILE *stream)

    # Random Number Functions
    void mpz_urandomb (mpz_t rop, gmp_randstate_t state, unsigned long int n)
    void mpz_urandomm (mpz_t rop, gmp_randstate_t state, mpz_t n)
    void mpz_rrandomb (mpz_t rop, gmp_randstate_t state, unsigned long int n)
    void mpz_random (mpz_t rop, mp_size_t max_size)
    void mpz_random2 (mpz_t rop, mp_size_t max_size)

    # Integer Import and Export
    void mpz_import (mpz_t rop, size_t count, int order, int size, int endian, size_t nails, void *op)
    void * mpz_export (void *rop, size_t *countp, int order, int size, int endian, size_t nails, mpz_t op)

    # Miscellaneous Functions
    bint mpz_fits_ulong_p (mpz_t op)
    bint mpz_fits_slong_p (mpz_t op)
    bint mpz_fits_uint_p (mpz_t op)
    bint mpz_fits_sint_p (mpz_t op)
    bint mpz_fits_ushort_p (mpz_t op)
    bint mpz_fits_sshort_p (mpz_t op)
    bint mpz_odd_p (mpz_t op)
    bint mpz_even_p (mpz_t op)
    size_t mpz_sizeinbase (mpz_t op, int base)

    # Special Functions
    void * _mpz_realloc (mpz_t integer, mp_size_t new_alloc)
    mp_limb_t mpz_getlimbn (mpz_t op, mp_size_t n)
    size_t mpz_size (mpz_t op)
